/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2008 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "xStatsInfo.h"

typedef BOOL (WINAPI *PGPI)(DWORD, DWORD, DWORD, DWORD, PDWORD);

#define PRODUCT_BUSINESS 0x00000006
#define PRODUCT_BUSINESS_N 0x00000010
#define PRODUCT_CLUSTER_SERVER 0x00000012
#define PRODUCT_DATACENTER_SERVER 0x00000008
#define PRODUCT_DATACENTER_SERVER_CORE 0x0000000C
#define PRODUCT_DATACENTER_SERVER_CORE_V 0x00000027
#define PRODUCT_DATACENTER_SERVER_V 0x00000025
#define PRODUCT_ENTERPRISE 0x00000004
#define PRODUCT_ENTERPRISE_N 0x0000001B
#define PRODUCT_ENTERPRISE_SERVER 0x0000000A
#define PRODUCT_ENTERPRISE_SERVER_CORE 0x0000000E
#define PRODUCT_ENTERPRISE_SERVER_CORE_V 0x00000029
#define PRODUCT_ENTERPRISE_SERVER_IA64 0x0000000F
#define PRODUCT_ENTERPRISE_SERVER_V 0x00000026
#define PRODUCT_HOME_BASIC 0x00000002
#define PRODUCT_HOME_BASIC_N 0x00000005
#define PRODUCT_HOME_PREMIUM 0x00000003
#define PRODUCT_HOME_PREMIUM_N 0x0000001A
#define PRODUCT_HOME_SERVER 0x00000013	
#define PRODUCT_MEDIUMBUSINESS_SERVER_MANAGEMENT 0x0000001E
#define PRODUCT_MEDIUMBUSINESS_SERVER_MESSAGING 0x00000020
#define PRODUCT_MEDIUMBUSINESS_SERVER_SECURITY 0x0000001F
#define PRODUCT_SERVER_FOR_SMALLBUSINESS 0x00000018
#define PRODUCT_SMALLBUSINESS_SERVER 0x00000009
#define PRODUCT_SMALLBUSINESS_SERVER_PREMIUM 0x00000019
#define PRODUCT_STANDARD_SERVER 0x00000007
#define PRODUCT_STANDARD_SERVER_CORE 0x0000000D
#define PRODUCT_STANDARD_SERVER_CORE_V 0x00000028
#define PRODUCT_STANDARD_SERVER_V 0x00000024
#define PRODUCT_STARTER 0x0000000B
#define PRODUCT_STORAGE_ENTERPRISE_SERVER 0x00000017
#define PRODUCT_STORAGE_EXPRESS_SERVER 0x00000014
#define PRODUCT_STORAGE_STANDARD_SERVER 0x00000015
#define PRODUCT_STORAGE_WORKGROUP_SERVER 0x00000016
#define PRODUCT_UNDEFINED 0x00000000
#define PRODUCT_ULTIMATE 0x00000001
#define PRODUCT_ULTIMATE_N 0x0000001C
#define PRODUCT_WEB_SERVER 0x00000011
#define PRODUCT_WEB_SERVER_CORE 0x0000001D

#define MMNODRV         //Installable driver support
#define MMNOSOUND       //Sound support
#define MMNOWAVE        //Waveform support
#define MMNOMIDI        //MIDI support
#define MMNOAUX         //Auxiliary audio support
//#define MMNOMIXER       //Mixer support
#define MMNOTIMER       //Timer support
#define MMNOJOY         //Joystick support
#define MMNOMCI         //MCI support
#define MMNOMMIO        //Multimedia file I/O support
#define MMNOMMSYSTEM    //General MMSYSTEM functions
#include "mmsystem.h"

#include <winsock.h>

// FIXME: Must be a better way to do this
#define DEFAULT_FILE_DATE_FORMAT "dd mmm yyyy hh:nn:ss"

#define _countof(array) (sizeof(array)/sizeof(array[0]))

const int unitsDefault = 0;
const int unitsBytes = 1;
const int unitsKB = 2;
const int unitsMB = 3;
const int unitsGB = 4;
const int unitsPercent = 5;

//    enum Bus
#define BusType     char
#define ISA         0
#define SMBus       1
#define VIA686Bus   2
#define DirectIO    3

//    enum SMB
#define SMBType         char
#define smtSMBIntel     0
#define smtSMBAMD       1
#define smtSMBALi       2
#define smtSMBNForce    3
#define smtSMBSIS       4

// enum Sensor Types
#define SensorType      char
#define stUnknown       0
#define stTemperature   1
#define stVoltage       2
#define stFan           3
#define stMhz           4
#define stPercentage    5  

#ifndef VK_VOLUME_MUTE
	#define VK_VOLUME_MUTE         0xAD
#endif

typedef struct {
    SensorType  iType;          // type of sensor
    int         Count;          // number of sensor for that type
} SharedIndex;

typedef struct {
    SensorType ssType;          // type of sensor
    unsigned char ssName[12];   // name of sensor
    char sspadding1[3];         // padding of 3 byte
    double ssCurrent;           // current value
    double ssLow;               // lowest readout
    double ssHigh;              // highest readout
    long ssCount;               // total number of readout
    char sspadding2[4];         // padding of 4 byte
    long double ssTotal;        // total amout of all readouts
    char sspadding3[6];         // padding of 6 byte
    double ssAlarm1;            // temp & fan: high alarm; voltage: % off;
    double ssAlarm2;            // temp: low alarm
} SharedSensor;

typedef struct {
    short siSMB_Base;            // SMBus base address
    BusType siSMB_Type;         // SMBus/Isa bus used to access chip
    SMBType siSMB_Code;         // SMBus sub type, Intel, AMD or ALi
    char siSMB_Addr;            // Address of sensor chip on SMBus
    unsigned char siSMB_Name[41];        // Nice name for SMBus
    short siISA_Base;            // ISA base address of sensor chip on ISA
    int siChipType;             // Chip nr, connects with Chipinfo.ini
    char siVoltageSubType;      // Subvoltage option selected
} SharedInfo;

typedef struct {
    double sdVersion;           // version number (example: 51090)
    SharedIndex sdIndex[10];     // Sensor index
    SharedSensor sdSensor[100];  // sensor info
    SharedInfo sdInfo;          // misc. info
    unsigned char sdStart[41];           // start time                              
    unsigned char sdCurrent[41];         // current time
    unsigned char sdPath[256];           // MBM path
} mbmSharedData;

xStatsInfo::xStatsInfo() :
	bCPUStarted(false),
	bNetStatsStarted(false),
	pCPUInfo(NULL),
	winampCurrentTrack(-1)
{
	IsOS(OS_2KXP);

	WSADATA wd;
	WSAStartup(0x0101, &wd);
	
	maskStack.push_back(true);

	shortWeekdays[0] = getLocaleInfo(LOCALE_SABBREVDAYNAME7);
	shortWeekdays[1] = getLocaleInfo(LOCALE_SABBREVDAYNAME1);
	shortWeekdays[2] = getLocaleInfo(LOCALE_SABBREVDAYNAME2);
	shortWeekdays[3] = getLocaleInfo(LOCALE_SABBREVDAYNAME3);
	shortWeekdays[4] = getLocaleInfo(LOCALE_SABBREVDAYNAME4);
	shortWeekdays[5] = getLocaleInfo(LOCALE_SABBREVDAYNAME5);
	shortWeekdays[6] = getLocaleInfo(LOCALE_SABBREVDAYNAME6);

	weekdays[0] = getLocaleInfo(LOCALE_SDAYNAME7);
	weekdays[1] = getLocaleInfo(LOCALE_SDAYNAME1);
	weekdays[2] = getLocaleInfo(LOCALE_SDAYNAME2);
	weekdays[3] = getLocaleInfo(LOCALE_SDAYNAME3);
	weekdays[4] = getLocaleInfo(LOCALE_SDAYNAME4);
	weekdays[5] = getLocaleInfo(LOCALE_SDAYNAME5);
	weekdays[6] = getLocaleInfo(LOCALE_SDAYNAME6);

	shortMonths[0] = getLocaleInfo(LOCALE_SABBREVMONTHNAME1);
	shortMonths[1] = getLocaleInfo(LOCALE_SABBREVMONTHNAME2);
	shortMonths[2] = getLocaleInfo(LOCALE_SABBREVMONTHNAME3);
	shortMonths[3] = getLocaleInfo(LOCALE_SABBREVMONTHNAME4);
	shortMonths[4] = getLocaleInfo(LOCALE_SABBREVMONTHNAME5);
	shortMonths[5] = getLocaleInfo(LOCALE_SABBREVMONTHNAME6);
	shortMonths[6] = getLocaleInfo(LOCALE_SABBREVMONTHNAME7);
	shortMonths[7] = getLocaleInfo(LOCALE_SABBREVMONTHNAME8);
	shortMonths[8] = getLocaleInfo(LOCALE_SABBREVMONTHNAME9);
	shortMonths[9] = getLocaleInfo(LOCALE_SABBREVMONTHNAME10);
	shortMonths[10] = getLocaleInfo(LOCALE_SABBREVMONTHNAME11);
	shortMonths[11] = getLocaleInfo(LOCALE_SABBREVMONTHNAME12);

	months[0] = getLocaleInfo(LOCALE_SMONTHNAME1);
	months[1] = getLocaleInfo(LOCALE_SMONTHNAME2);
	months[2] = getLocaleInfo(LOCALE_SMONTHNAME3);
	months[3] = getLocaleInfo(LOCALE_SMONTHNAME4);
	months[4] = getLocaleInfo(LOCALE_SMONTHNAME5);
	months[5] = getLocaleInfo(LOCALE_SMONTHNAME6);
	months[6] = getLocaleInfo(LOCALE_SMONTHNAME7);
	months[7] = getLocaleInfo(LOCALE_SMONTHNAME8);
	months[8] = getLocaleInfo(LOCALE_SMONTHNAME9);
	months[9] = getLocaleInfo(LOCALE_SMONTHNAME10);
	months[10] = getLocaleInfo(LOCALE_SMONTHNAME11);
	months[11] = getLocaleInfo(LOCALE_SMONTHNAME12);
}

xStatsInfo::~xStatsInfo()
{
	if (bNetStatsStarted)
	{
		for(NetStatsListIterator nsit = netStatsList.begin(); nsit != netStatsList.end(); nsit++)
		{
			(*nsit)->Stop();
			delete *nsit;
			*nsit = NULL;
		}
		netStatsList.clear();
	}

	if (pCPUInfo)
	{
		delete pCPUInfo;
		pCPUInfo = NULL;
	}

	if (bCPUStarted)
		endProcessorStatistics();

	WSACleanup();

	int i = 0;

	for(i = 0; i < 7; i++)
	{
		delete[] shortWeekdays[i];
		delete[] weekdays[i];
	}

	for(i = 0; i < 12; i++)
	{
		delete[] shortMonths[i];
		delete[] months[i];
	}
}

string xStatsInfo::processFinalText(const string &text, bool *dynamic)
{
	string output0 = "";
	
	bool execute = false;
	int length = text.length();
	int i = 0;
	
	//Set to initially to False!!
	if (dynamic) *dynamic = false;
	
	while(i < length)
	{
		if(text[i] == '[')
		{
			i++;
			
			if(i < length && text[i] == '[')
			{
				if(maskStack.back()) output0.append(1, '[');
				i++;
				continue;
			}
			else if(i < length && text[i] == '!')
			{
				if(maskStack.back()) output0.append("[!");
				execute = true;
				i++;
				continue;
			}
			
			string temp = process(text, i, length, dynamic);
			if(maskStack.back()) output0.append(temp);

			if(i >= length || text[i++] != ']')
			{
				output0 = "[syntax error]";
				break;
			}
			continue;
		}
		else if(text[i] == ']' && !execute)
		{
			i++;
			
			if(i < length && text[i] == ']')
			{
				if(maskStack.back()) output0.append(1, ']');
				i++;
				continue;
			}
		}
		else if (text[i] == ']' && execute)
		{
			execute = false;
		}
		
		if(maskStack.back()) output0.append(1, text[i]);
		i++;
	}
	
	length = output0.length();
	i = 0;
	
	return output0;
}

string xStatsInfo::evaluateFunction(const string &functionName, const vector<string> &arguments, bool *dynamic = 0)
{
	string name = lowerCase(functionName);

	if(name == "if" && arguments.size() >= 1)
	{
		doIf(arguments);
		return "";
	}
	else if(name == "ifeval" && arguments.size() >= 2)
	{
		doIfEval(arguments);
		return "";
	}
	else if(name == "elseif" && arguments.size() >= 1)
	{
		doElseIf(arguments);
		return "";
	}
	else if(name == "elseifeval" && arguments.size() >= 2)
	{
		doElseIfEval(arguments);
		return "";
	}
	else if(name == "else")
	{
		doElse();
		return "";
	}
	else if(name == "endif")
	{
		doEndIf();
		return "";
	}

	if(name == "activetask" && arguments.size() >= 0)
		return getActiveTask(arguments, dynamic);
	else if(name == "tasks" && arguments.size() >= 0)
		return getTasks(arguments, dynamic);
	else if(name == "mousepos")
		return getMousePos(dynamic);
	else if(name == "battery")
		return getBattery(dynamic);
	else if(name == "computername")
		return getComputerName(dynamic);
	else if(name == "kblayout")
		return getKeyboardLayoutName(dynamic);
	else if(name == "skblayout")
		return getKeyboardLayoutShortName(dynamic);
	else if(name == "keyboardstate")
		return getKeyboardState(arguments, dynamic);
	else if(name == "connected")
		return getConnected(dynamic);
	else if(name == "cpu")
		return getCPU(arguments, dynamic);
	else if(name == "cpuspeed")
		return getCPUSpeed(arguments, dynamic);
	else if(name == "cpuinfo")
		return getCPUInfo(arguments, dynamic);
	else if(name == "date")
		return getDate(arguments, dynamic);
	else if(name == "datecreated")
		return getDateCreated(arguments, dynamic);
	else if(name == "datelastmodified")
		return getDateLastModified(arguments, dynamic);
	else if(name == "diskavailable" && arguments.size() >= 1)
		return getDiskAvailable(arguments, dynamic);
	else if(name == "diskinuse" && arguments.size() >= 1)
		return getDiskInUse(arguments, dynamic);
	else if(name == "disktotal" && arguments.size() >= 1)
		return getDiskTotal(arguments, dynamic);
	else if(name == "fileexists" && arguments.size() >= 1)
		return getFileExists(arguments[0], dynamic);
	else if(name == "firstline" && arguments.size() >= 1)
		return getFirstLine(arguments, dynamic);
	else if(name == "hostname")
		return getHostName(dynamic);
	else if(name == "ip")
		return getIP(arguments, dynamic);
	else if(name == "online" && arguments.size() == 2)
		return getOnline(arguments, dynamic);
	else if(name == "itime")
		return getInternetTime(dynamic);
	else if(name == "lastline" && arguments.size() >= 1)
		return getLastLine(arguments, dynamic);
	else if(name == "linecount" && arguments.size() >= 1)
		return getLineCount(arguments[0], dynamic);
	else if(name == "line")
		return getLine(arguments, dynamic);
	else if(name == "matchline")
		return getMatchLine(arguments, dynamic);
	else if(name == "mbmtemperature")
		return getMBMData(stTemperature, arguments, dynamic);
	else if(name == "mbmcpuusage")
		return getMBMData(stPercentage, arguments, dynamic);
	else if(name == "mbmfanspeed")
		return getMBMData(stFan, arguments, dynamic);
	else if(name == "mbmvoltage")
		return getMBMData(stVoltage, arguments, dynamic);
	else if(name == "mbmloaded")
		return getMBMLoaded(dynamic);
	else if(name == "memavailable")
		return getMemAvailable(arguments, dynamic);
	else if(name == "meminuse")
		return getMemInUse(arguments, dynamic);
	else if(name == "memtotal")
		return getMemTotal(arguments, dynamic);
	else if(name == "netadaptername")
		return getNetAdapterName(arguments, dynamic);
	else if(name == "netin")
		return getNetIn(arguments, dynamic);
	else if(name == "netout")
		return getNetOut(arguments, dynamic);
	else if(name == "netinout")
		return getNetInOut(arguments, dynamic);
	else if(name == "nettotalin")
		return getNetTotalIn(arguments, dynamic);
	else if(name == "nettotalout")
		return getNetTotalOut(arguments, dynamic);
	else if(name == "nettotalinout")
		return getNetTotalInOut(arguments, dynamic);
	else if(name == "os")
		return getOS(dynamic);
	else if(name == "osex")
		return getOSEx(dynamic);
	else if(name == "powersource")
		return getPowerSource(arguments, dynamic);
	else if(name == "randomline" && arguments.size() >= 1)
		return getRandomLine(arguments[0], dynamic);
	else if(name == "size")
		return getSize(arguments, dynamic);
	else if(name == "swapavailable")
		return getSwapAvailable(arguments, dynamic);
	else if(name == "swapinuse")
		return getSwapInUse(arguments, dynamic);
	else if(name == "swaptotal")
		return getSwapTotal(arguments, dynamic);
	else if(name == "time")
		return getTime(arguments, dynamic);
	else if(name == "uptime")
		return getUptime(arguments, dynamic);
	else if(name == "username")
		return getUserName(dynamic);
	else if (name == "volume" && arguments.size() >= 0)
		return getVolume(arguments, dynamic);
	else if (name == "mute")
		return getMute(dynamic);
	else if(name == "winampsong" && arguments.size() >= 0)
		return getWinampSong(arguments, dynamic);
	else if(name == "winampartist" && arguments.size() >= 0)
		return getWinampArtist(arguments, dynamic);
	else if(name == "winamptitle" && arguments.size() >= 0)
		return getWinampTitle(arguments, dynamic);
	else if(name == "winamptime" && arguments.size() >= 0)
		return getWinampTime(arguments, dynamic);
	else if(name == "winampremaintime" && arguments.size() >= 0)
		return getWinampRemainTime(arguments, dynamic);
	else if(name == "winamptotaltime" && arguments.size() >= 0)
		return getWinampTotalTime(arguments, dynamic);
	else if(name == "winampstatus" && arguments.size() >= 0)
		return getWinampStatus(arguments, dynamic);
	else if(name == "winampbitrate")
		return getWinampBitRate(dynamic);
	else if(name == "winampsamplerate")
		return getWinampSampleRate(dynamic);
	else if(name == "winampplaying")
		return getWinampPlaying(dynamic);
	else if(name == "winamppaused")
		return getWinampPaused(dynamic);
	else if(name == "winampstopped")
		return getWinampStopped(dynamic);
	else if(name == "winamprepeat")
		return getWinampRepeat(dynamic);
	else if(name == "winampshuffle")
		return getWinampShuffle(dynamic);
	// .alur
	else if(name == "winamprating")
		return getWinampRating(dynamic);
    else if(name == "winampalbum")
		return getWinampInfo(0, dynamic);
    else if(name == "winampgenre")
		return getWinampInfo(1, dynamic);
    else if(name == "winampalbumtrack")
		return getWinampInfo(2, dynamic);
    else if(name == "winampyear")
		return getWinampInfo(3, dynamic);
    else if(name == "winampcomment")
		return getWinampInfo(4, dynamic);
	// .alur
	else if(name == "windowtitle" && arguments.size() >= 1)
		return getWindowTitle(arguments[0], dynamic);
	else if (name == "clipboardtext")
		return getClipboardText(dynamic);
	else if(name == "hideifempty" && arguments.size() >= 1)
		return arguments[0];
	else if(name == "exportedevar" && arguments.size() >= 1)
		return exportedEvars(arguments[0], dynamic);
	else if(name == "empty" && arguments.size() == 1)
		return empty(arguments[0], dynamic);
	else if(name == "notempty" && arguments.size() == 1)
		return notEmpty(arguments[0], dynamic);
	else if(name == "after" && arguments.size() == 2)
		return after(arguments[0], arguments[1]);
	else if(name == "afterlast" && arguments.size() == 2)
		return afterLast(arguments[0], arguments[1]);
	else if(name == "before" && arguments.size() == 2)
		return before(arguments[0], arguments[1]);
	else if(name == "beforelast" && arguments.size() == 2)
		return beforeLast(arguments[0], arguments[1]);
	else if(name == "between" && arguments.size() >= 3)
		return between(arguments[0], arguments[1], arguments[2], (arguments.size() == 4 ? arguments[3] : ""));
	else if(name == "capitalize" && arguments.size() == 1)
		return capitalize(arguments[0]);
	else if(name == "lowercase" && arguments.size() == 1)
		return lowerCase(arguments[0]);
	else if(name == "trim" && arguments.size() == 1)
		return trim(arguments[0]);
	else if(name == "trim" && arguments.size() == 2)
		return trim(arguments[0], arguments[1]);
	else if(name == "remove" && arguments.size() == 2)
		return remove(arguments[0], arguments[1]);
	else if(name == "replace" && arguments.size() >= 2)
		return replace(arguments[0], arguments[1], (arguments.size() == 3 ? arguments[2] : ""));
	else if(name == "uppercase" && arguments.size() == 1)
		return upperCase(arguments[0]);
	else if(name == "verticaldown" && arguments.size() == 1)
		return verticaldown(arguments[0]);
	else if(name == "verticalup" && arguments.size() == 1)
		return verticalup(arguments[0]);
	else if(name == "bytes") // units hack
		return "1";
	else if(name == "kb")
		return "2";
	else if(name == "mb")
		return "3";
	else if(name == "gb")
		return "4";
	else if(name == "%")
		return "5";
	else if(name == "semicolon") // hack to avoid escaping ;'s
		return ";";
	else if(name == "dollar") // hack to avoid $
		return "$";

	return name;
}

string xStatsInfo::process(const string &text, int &i, int length, bool *dynamic = NULL)
{
	// skip whitespace
	while(i < length && isspace(text[i]))
		i++;
	
	// is it a string literal?
	if(i < length && (text[i] == '\"' || text[i] == '\''))
	{
		string literalValue;
		char quoteChar = text[i++];
		
		while(i < length && text[i] != quoteChar)
			literalValue.append(1, text[i++]);
		
		if(i < length) i++;
		return literalValue;
	}
	else if(i < length && isdigit(text[i])) // is it a numeric constant?
	{
		string number;

		while(i < length && isdigit(text[i]))
			number.append(1, text[i++]);

		return number;
	}
	else // otherwise its a name
	{
		string name;
		vector<string> arguments;
		
		// retrieve name
		while(i < length && (isalnum(text[i]) || text[i] == '-' || text[i] == '%'))
			name.append(1, text[i++]);
		
		// skip whitespace
		while(i < length && isspace(text[i]))
			i++;
		
		// has argument list?
		if(i < length && text[i] == '(')
		{
			// process each argument
			do
			{
				i++;

				// call recursively to handle
				arguments.push_back(process(text, i, length, dynamic));
				
				// skip whitespace
				while(i < length && isspace(text[i]))
					i++;
			}
			while(i < length && text[i] == ',');

			// must be a closing parenthesis
			if(i >= length || text[i++] != ')')
				return "[error]";
		}
		
		// get it
		return evaluateFunction(name, arguments, dynamic);
	}
}

void xStatsInfo::doIf(const vector<string> &arguments)
{
	int value1 = 1;
	int value2 = 1;
	int value3 = 1;
	if (arguments.size() == 1)
	{
		value1 = atoi(arguments[0].c_str());
	}
	else if(arguments.size() == 2)
	{
		value1 = atoi(arguments[0].c_str());
		value2 = atoi(arguments[1].c_str());
	}
	else if(arguments.size() == 3)
	{
		value1 = atoi(arguments[0].c_str());
		value2 = atoi(arguments[1].c_str());
		value3 = atoi(arguments[2].c_str());
	}
	
	condStack.push_back( (value1 != 0 && value2 != 0 && value3 != 0) );
	depthStack.push_back(1);
	maskStack.push_back(maskStack.back() && condStack.back());
}

void xStatsInfo::doIfEval(const vector<string> &arguments)
{
	int result = 0;
	if(arguments.size() == 2)
	{
		if ( _stricmp(arguments[0].c_str(), arguments[1].c_str()) == NULL )
			result = 1;
	}
	else if (arguments.size() == 3)
	{
		if ( _stricmp(arguments[1].c_str(), "=") == NULL )
		{
			if ( _stricmp(arguments[0].c_str(), arguments[2].c_str()) == NULL )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), "<") == NULL )
		{
			if ( atoi(arguments[0].c_str()) < atoi(arguments[2].c_str()) )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), ">") == NULL )
		{
			if ( atoi(arguments[0].c_str()) > atoi(arguments[2].c_str()) )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), "<=") == NULL )
		{
			if ( atoi(arguments[0].c_str()) <= atoi(arguments[2].c_str()) )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), ">=") == NULL )
		{
			if ( atoi(arguments[0].c_str()) >= atoi(arguments[2].c_str()) )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), "<>") == NULL )
		{
			if ( _stricmp(arguments[0].c_str(), arguments[2].c_str()) != NULL)
				result = 1;
		}
	}
	
	condStack.push_back( result == 1 );
	depthStack.push_back(1);
	maskStack.push_back(maskStack.back() && condStack.back());
}

void xStatsInfo::doElseIf(const vector<string> &arguments)
{
	if(maskStack.empty())
		return;

	int value1 = 1;
	int value2 = 1;
	int value3 = 1;
	if (arguments.size() == 1)
	{
		value1 = atoi(arguments[0].c_str());
	}
	else if(arguments.size() == 2)
	{
		value1 = atoi(arguments[0].c_str());
		value2 = atoi(arguments[1].c_str());
	}
	else if(arguments.size() == 3)
	{
		value1 = atoi(arguments[0].c_str());
		value2 = atoi(arguments[1].c_str());
		value3 = atoi(arguments[2].c_str());
	}

	maskStack.pop_back();
	maskStack.push_back(maskStack.back() && !condStack.back());
	condStack.push_back( (value1 != 0 && value2 != 0 && value3 != 0) );
	depthStack.back()++;
	maskStack.push_back(maskStack.back() && condStack.back());
}

void xStatsInfo::doElseIfEval(const vector<string> &arguments)
{
	if(maskStack.empty())
		return;

	int result = 0;
	if(arguments.size() == 2)
	{
		if ( _stricmp(arguments[0].c_str(), arguments[1].c_str()) == NULL )
			result = 1;
	}
	else if (arguments.size() == 3)
	{
		if ( _stricmp(arguments[1].c_str(), "=") == NULL )
		{
			if ( _stricmp(arguments[0].c_str(), arguments[2].c_str()) == NULL )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), "<") == NULL )
		{
			if ( atoi(arguments[0].c_str()) < atoi(arguments[2].c_str()) )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), ">") == NULL )
		{
			if ( atoi(arguments[0].c_str()) > atoi(arguments[2].c_str()) )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), "<=") == NULL )
		{
			if ( atoi(arguments[0].c_str()) <= atoi(arguments[2].c_str()) )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), ">=") == NULL )
		{
			if ( atoi(arguments[0].c_str()) >= atoi(arguments[2].c_str()) )
				result = 1;
		}
		else if ( _stricmp(arguments[1].c_str(), "<>") == NULL )
		{
			if ( _stricmp(arguments[0].c_str(), arguments[2].c_str()) != NULL)
				result = 1;
		}
	}
	
	maskStack.pop_back();
	maskStack.push_back(maskStack.back() && !condStack.back());
	condStack.push_back( result == 1 );
	depthStack.back()++;
	maskStack.push_back(maskStack.back() && condStack.back());
}

void xStatsInfo::doElse()
{
	if(maskStack.empty())
		return;

	maskStack.pop_back();
	maskStack.push_back(maskStack.back() && !condStack.back());
}

void xStatsInfo::doEndIf()
{
	if(depthStack.empty())
		return;

	int depth = depthStack.back();
	depthStack.pop_back();

	while(depth--)
	{
		condStack.pop_back();
		maskStack.pop_back();
	}
}

string xStatsInfo::getVolume(const vector<string> &arguments, bool *dynamic)
{
	bool winampVol = false;
	int mixerline = 0;
	if (arguments.size() != 0)
	{
		if (_stricmp (arguments[0].c_str(), "winamp") == NULL)
			winampVol = true;
		else
			mixerline = atoi( arguments[0].c_str() );
	}

	CString mainVolume = "";
	double volume = 0.0;

	if ( !winampVol )
	{
		DWORD                           dwVolume = -1;
		MMRESULT                        result;
		HMIXER                          hMixer;
		MIXERLINE                       ml   = {0};
		MIXERLINECONTROLS               mlc  = {0};
		MIXERCONTROL                    mc   = {0};
		MIXERCONTROLDETAILS             mcd  = {0};
		MIXERCONTROLDETAILS_UNSIGNED    mcdu = {0};

		// get a handle to the mixer device
		result = mixerOpen(&hMixer, 0, 0, 0, MIXER_OBJECTF_HMIXER);
		if (MMSYSERR_NOERROR == result)
		{
			ml.cbStruct        = sizeof(MIXERLINE);
			ml.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;

			// get the speaker line of the mixer device
			result = mixerGetLineInfo((HMIXEROBJ) hMixer, &ml, MIXER_GETLINEINFOF_COMPONENTTYPE);
			if (MMSYSERR_NOERROR == result)
			{
				mlc.cbStruct      = sizeof(MIXERLINECONTROLS);
				mlc.dwLineID      = ml.dwLineID;
				mlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
				mlc.cControls     = 1;
				mlc.pamxctrl      = &mc;
				mlc.cbmxctrl      = sizeof(MIXERCONTROL);

				// get the volume controls associated with the speaker line
				result = mixerGetLineControls((HMIXEROBJ) hMixer, &mlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
				if (MMSYSERR_NOERROR == result)
				{
					mcd.cbStruct    = sizeof(MIXERCONTROLDETAILS);
					mcd.hwndOwner   = 0;
					mcd.dwControlID = mc.dwControlID;
					mcd.paDetails   = &mcdu;
					mcd.cbDetails   = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
					mcd.cChannels   = 1;

					// get the volume
					result = mixerGetControlDetails((HMIXEROBJ) hMixer, &mcd, MIXER_GETCONTROLDETAILSF_VALUE);
					if (MMSYSERR_NOERROR == result)
					{
						dwVolume = mcdu.dwValue;
						
						//NO Error -> Use Dynamic
						if (dynamic) *dynamic = true;
					}
				}
			}
    
			mixerClose(hMixer); 
			
			volume = dwVolume;
			volume = volume/655.35;
		}
	}
	else
	{
		HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

		if(hwndWA2 != NULL)
		{
			volume = SendMessage(hwndWA2, WM_USER, -666, 122);
			volume = (volume*100)/255;
		}
		
		if (dynamic) *dynamic = true;
	}
	
	mainVolume.Format("%.1f", volume);
	mainVolume += " %";

	return (string) mainVolume;
}

string xStatsInfo::getMute(bool *dynamic)
{
	LONG							lVal = -1;
	MMRESULT                        result;
	HMIXER                          hMixer;
	MIXERLINE                       ml   = {0};
	MIXERLINECONTROLS               mlc  = {0};
	MIXERCONTROL                    mc   = {0};
	MIXERCONTROLDETAILS             mcd  = {0};
	MIXERCONTROLDETAILS_BOOLEAN		mcdu = {0};

	// get a handle to the mixer device
	result = mixerOpen(&hMixer, 0, 0, 0, MIXER_OBJECTF_HMIXER);
	if (MMSYSERR_NOERROR == result)
	{
		ml.cbStruct        = sizeof(MIXERLINE);
		ml.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;

		// get the speaker line of the mixer device
		result = mixerGetLineInfo((HMIXEROBJ) hMixer, &ml, MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE);
		if (MMSYSERR_NOERROR == result)
		{
			mlc.cbStruct      = sizeof(MIXERLINECONTROLS);
			mlc.dwLineID      = ml.dwLineID;
			mlc.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
			mlc.cControls     = 1;
			mlc.pamxctrl      = &mc;
			mlc.cbmxctrl      = sizeof(MIXERCONTROL);

			// get the volume controls associated with the speaker line
			result = mixerGetLineControls((HMIXEROBJ) hMixer, &mlc, MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE);
			if (MMSYSERR_NOERROR == result)
			{
				mcd.cbStruct    = sizeof(MIXERCONTROLDETAILS);
				mcd.hwndOwner   = 0;
				mcd.dwControlID = mc.dwControlID;
				mcd.paDetails   = &mcdu;
				mcd.cbDetails   = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
				mcd.cChannels   = 1;

				// get the volume
				result = mixerGetControlDetails((HMIXEROBJ) hMixer, &mcd, MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE);
				if (MMSYSERR_NOERROR == result)
				{
					lVal = mcdu.fValue;	
					
					//NO Error -> Use Dynamic
					if (dynamic) *dynamic = true;
				}
			}
		}
    
		mixerClose(hMixer); 
	}

	return (lVal != 0 ? "1" : "0");
}


string xStatsInfo::getCPUSpeed(const vector<string> &arguments, bool *dynamic)
{
	if (pCPUInfo == NULL)
		pCPUInfo = new CPUInfo();
	
	string result = "? MHz";

	if (arguments.size() != 0)
	{
		if (_stricmp(arguments[0].c_str(), "dynamic") == NULL)
		{
			result = IntToString(pCPUInfo->GetProcessorClockFrequency(true));
			if (dynamic) *dynamic = true;
		}
		else
		{
			result = IntToString(pCPUInfo->GetProcessorClockFrequency());
		}
 
		result.append(" MHz");
	}
	else
	{
		result = IntToString(pCPUInfo->GetProcessorClockFrequency());
		result.append(" MHz");
	}

	return result;
}

string xStatsInfo::getCPUInfo(const vector<string> &arguments, bool *dynamic)
{
	if (pCPUInfo == NULL)
		pCPUInfo = new CPUInfo();

	string result = "";

	if (pCPUInfo->DoesCPUSupportCPUID())
	{
		if (arguments.size() != 0)
		{
			if (_stricmp(arguments[0].c_str(), "vendorid") == NULL)
				result.append(pCPUInfo->GetVendorID());
			else if (_stricmp(arguments[0].c_str(), "typeid") == NULL)
				result.append(pCPUInfo->GetTypeID());
			else if (_stricmp(arguments[0].c_str(), "familyid") == NULL)
				result.append(pCPUInfo->GetFamilyID());
			else if (_stricmp(arguments[0].c_str(), "modelid") == NULL)
				result.append(pCPUInfo->GetModelID());
			else if (_stricmp(arguments[0].c_str(), "steppingcode") == NULL)
				result.append(pCPUInfo->GetSteppingCode());
			else if (_stricmp(arguments[0].c_str(), "brandid") == NULL)
				result.append(pCPUInfo->GetExtendedProcessorName());
		}
		else
		{
			result.append("Vendor ID: ");
			result.append(pCPUInfo->GetVendorID());
			result.append("<br>CPU Type ID: ");
			result.append(pCPUInfo->GetTypeID());
			result.append("<br>Family ID: ");
			result.append(pCPUInfo->GetFamilyID());
			result.append("<br>Model ID: ");
			result.append(pCPUInfo->GetModelID());
			result.append("<br>Stepping Code: ");
			result.append(pCPUInfo->GetSteppingCode());
			result.append("<br>Brand ID: ");
			result.append(pCPUInfo->GetExtendedProcessorName());
		}
	}
	else
	{
		result = "Error: CPU does not support CPUID!";
	}

	return result;
}

string xStatsInfo::getActiveTask(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	string result = UpdateTasks("", true);
	if ( !result.empty() )
	{
		return result;
	}
	else
	{
		if (arguments.size() != 0)
			return arguments[0];
		else
			return "";
	}
}

string xStatsInfo::getTasks(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	if (arguments.size() != 0)
		return UpdateTasks(arguments[0]);
	else
		return UpdateTasks(" - ");
}

string xStatsInfo::getMousePos(bool *dynamic)
{
	if (dynamic) *dynamic = true;
	
	POINT pointCursor;
	GetCursorPos(&pointCursor);

	return ( "x: " + IntToString(pointCursor.x) + ", y: " + IntToString(pointCursor.y) );
}

string xStatsInfo::getComputerName(bool *dynamic)
{
	char buffer[32] = {0};
	unsigned long bufferLen = 32;

	GetComputerName(buffer, &bufferLen);
	return string(buffer);
}

string xStatsInfo::getKeyboardLayoutName(bool *dynamic)
{
	char buffer[128] = {0};

	DWORD idthd=GetWindowThreadProcessId(GetForegroundWindow(),NULL);
	GetLocaleInfo(MAKELCID(LOWORD(GetKeyboardLayout(idthd)), SORT_DEFAULT),LOCALE_SLANGUAGE,buffer,128);
	return string(buffer);
}

string xStatsInfo::getKeyboardLayoutShortName(bool *dynamic)
{
	char buffer[128] = {0};

	DWORD idthd = GetWindowThreadProcessId(GetForegroundWindow(),NULL);
	GetLocaleInfo(MAKELCID(LOWORD(GetKeyboardLayout(idthd)), SORT_DEFAULT),LOCALE_SISO639LANGNAME,buffer,128);
	return string(buffer);

}

string xStatsInfo::getKeyboardState(const vector<string> &arguments, bool *dynamic)
{
	if (arguments.size() >= 1)
	{
		if (dynamic) *dynamic = true;

		if ( _stricmp(arguments[0].c_str(), "caps lock") == NULL || _stricmp(arguments[0].c_str(), "capslock") == NULL )
		{
			if ( (int) (short) GetKeyState( VK_CAPITAL ) == 1 )
				return "1";
			else
				return "0";
		}
		else if ( _stricmp(arguments[0].c_str(), "num lock") == NULL || _stricmp(arguments[0].c_str(), "numlock") == NULL )
		{
			if ( (int) (short) GetKeyState( VK_NUMLOCK ) == 1 )
				return "1";
			else
				return "0";
		}
		else if ( _stricmp(arguments[0].c_str(), "scroll lock") == NULL || _stricmp(arguments[0].c_str(), "scrolllock") == NULL )
		{
			if ( (int) (short) GetKeyState( VK_SCROLL ) == 1 )
				return "1";
			else
				return "0";
		}
	}
	
	return "0";
}

string xStatsInfo::getConnected(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	if (!bNetStatsStarted)
	{
		NetStats *netStats = new NetStats(0);
		
		bNetStatsStarted = true;
		netStats->Start();
		
		netStatsList.insert(netStatsList.end(), netStats);
	}

	return (*netStatsList.begin())->IsConnAvailable() ? "1" : "0";
}

string xStatsInfo::getCPU(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;
	
	if (!bCPUStarted)
	{
		startProcessorStatistics(IsOS(OS_NT), IsOS(OS_2KXP));
		bCPUStarted = true;	
	}

	if (arguments.size() >= 1)
		return (IntToString( getProcessorUsage(arguments[0]) ));
	else
		return (IntToString( getProcessorUsage() ));
}

string xStatsInfo::getDate(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	SYSTEMTIME st;
	getLocalizedTime(&st, arguments.size() >= 2 ? arguments[1] : "");

	if(arguments.size() >= 1)
	{
		return formatDateTime(arguments[0], st, 0);
	}
	else
	{
		char buffer[64] = {0};
		GetDateFormat(LOCALE_USER_DEFAULT, DATE_LONGDATE, &st, 0, buffer, 64);
		return string(buffer);
	}
}

string xStatsInfo::getInternetTime(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	SYSTEMTIME st;
	GetSystemTime(&st);

	int seconds = ((((st.wHour + 1) % 24) * 60) + st.wMinute) * 60 + st.wSecond;
	int beats = (1000 * seconds) / 86400;

	char output[8] = {0};
	StringCchPrintf(output, 8, "@%03d", beats);
	return (string) output;
}

string xStatsInfo::getDiskAvailable(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;
	
	largeInt freeBytes;
	largeInt totalBytes;
	int units = unitsDefault;

	if (arguments.size() >= 2)
		units = atoi(arguments[1].c_str());

	getDiskFreeSpace(arguments[0], freeBytes, totalBytes);
	
	return formatByteSize(freeBytes, totalBytes, units);
}

string xStatsInfo::getDiskInUse(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	largeInt freeBytes;
	largeInt totalBytes;
	int units = unitsDefault;

	if (arguments.size() >= 2)
		units = atoi(arguments[1].c_str());

	getDiskFreeSpace(arguments[0], freeBytes, totalBytes);
	
	return formatByteSize(totalBytes - freeBytes, totalBytes, units);
}

string xStatsInfo::getDiskTotal(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;
	
	largeInt freeBytes;
	largeInt totalBytes;
	int units = unitsDefault;

	if (arguments.size() >= 2)
		units = atoi(arguments[1].c_str());

	getDiskFreeSpace(arguments[0], freeBytes, totalBytes);
	
	return formatByteSize(totalBytes, totalBytes, units);
}

string xStatsInfo::getHostName(bool *dynamic)
{
	char hostName[256] = {0};
	gethostname(hostName, 256);
	hostent *hostInfo = gethostbyname(hostName);

	if(hostInfo)
		return (string) hostInfo->h_name;
	else
		return "";
}

string xStatsInfo::getIP(const vector<string> &arguments, bool *dynamic)
{
	char hostName[256];
	gethostname(hostName, 256);
	hostent *hostInfo = gethostbyname(hostName);

	if(!hostInfo)
		return "";

	int numIPs = 0;

	while(hostInfo->h_addr_list[numIPs])
		numIPs++;

	if(numIPs == 0)
		return "";

	int n = 0;
	int value = 0;

	if(arguments.size() >= 1)
		n = atoi(arguments[0].c_str()) - 1;

	if(n < 0)
		n = 0;
	if(n >= numIPs)
		n = numIPs - 1;

	return string(inet_ntoa(*((in_addr *) hostInfo->h_addr_list[n])));
}

string xStatsInfo::getOnline(const vector<string> &arguments, bool *dynamic)
{
	if (!bNetStatsStarted)
	{
		NetStats *netStats = new NetStats(0);
		
		bNetStatsStarted = true;
		netStats->Start();
		
		netStatsList.insert(netStatsList.end(), netStats);
	}

	if ( (*netStatsList.begin())->IsConnAvailable() && arguments.size() == 2 )
	{
		static char	ip[16] = {0};
		char buff[4096] = {0};
		char *p = NULL;
		int sock = -1;
		struct sockaddr_in sa;
		struct hostent *h = NULL;
		char req[2048] = {0};
	
		strcpy(req, "GET ");
		strcat(req, arguments[1].c_str());
		strcat(req, " HTTP/1.0\r\n"
						"Accept: text/plain\r\n"
						"Connection: close\r\n"
						"Host: ");
		strcat(req, arguments[0].c_str());
		strcat(req, "\r\n\r\n");
	
		memset(ip, 0, sizeof ip);
		memset(buff, 0, sizeof buff);
		memset(&sa, 0, sizeof sa);

		if (!(h = gethostbyname( arguments[0].c_str() )))
			return "ERROR: Failed to determine host IP address.";
		else
			if ((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1)
				return "ERROR: Failed to allocate socket.";

		sa.sin_family		= AF_INET;
		sa.sin_port			= htons(80);
		sa.sin_addr.s_addr	= *((ULONG *) h->h_addr_list[0]);

		if (connect(sock, (struct sockaddr *) &sa, sizeof sa) == -1)
			return closesocket(sock), "ERROR: Failed to connect to server.";
		else
		if (send(sock, req, strlen(req), 0) != (int) strlen(req))
			return closesocket(sock), "ERROR: Failed to send HTTP request.";
		else
		if (recv(sock, buff, (sizeof buff)-5, 0) <= 0)
			return closesocket(sock), "ERROR: Server failed to respond.";

		closesocket(sock);

		if ( p = strstr(buff, "\r\n\r\n") )
		{
			CString onlineDoc = p+4;
				
			onlineDoc.Replace("\r\n", "<br>");
			onlineDoc.Replace("\n", "<br>");

			return (string) onlineDoc;
		}
		else
		{
			return (string) "N/A";
		}
	}
	else
	{
		return (string) "N/A";
	}
}

string xStatsInfo::getMemAvailable(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	int units = unitsDefault;

	if (arguments.size() >= 1)
		units = atoi(arguments[0].c_str());

	if ( IsOS(OS_2KXP) )
	{
		MEMORYSTATUSEX ms;
		ms.dwLength = sizeof(MEMORYSTATUSEX);
		GlobalMemoryStatusEx(&ms);

		return formatByteSize(ms.ullAvailPhys, ms.ullTotalPhys, units);
	}
	else
	{
		MEMORYSTATUS ms;
		ms.dwLength = sizeof(MEMORYSTATUS);
		GlobalMemoryStatus(&ms);

		return formatByteSize(ms.dwAvailPhys, ms.dwTotalPhys, units);
	}
}

string xStatsInfo::getMemInUse(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	int units = unitsDefault;

	if(arguments.size() >= 1)
		units = atoi(arguments[0].c_str());

	if ( IsOS(OS_2KXP) )
	{
		MEMORYSTATUSEX ms;
		ms.dwLength = sizeof(MEMORYSTATUSEX);
		GlobalMemoryStatusEx(&ms);

		return formatByteSize(ms.ullTotalPhys - ms.ullAvailPhys, ms.ullTotalPhys, units);
	}
	else
	{
		MEMORYSTATUS ms;
		ms.dwLength = sizeof(MEMORYSTATUS);
		GlobalMemoryStatus(&ms);

		return formatByteSize(ms.dwTotalPhys - ms.dwAvailPhys, ms.dwTotalPhys, units);
	}
}

string xStatsInfo::getMemTotal(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	int units = unitsDefault;

	if (arguments.size() >= 1)
		units = atoi(arguments[0].c_str());

	if ( IsOS(OS_2KXP) )
	{
		MEMORYSTATUSEX ms;
		ms.dwLength = sizeof(MEMORYSTATUSEX);
		GlobalMemoryStatusEx(&ms);

		return formatByteSize(ms.ullTotalPhys, ms.ullTotalPhys, units);
	}
	else
	{
		MEMORYSTATUS ms;
		ms.dwLength = sizeof(MEMORYSTATUS);
		GlobalMemoryStatus(&ms);

		return formatByteSize(ms.dwTotalPhys, ms.dwTotalPhys, units);
	}
}

string xStatsInfo::getOS(bool *dynamic)
{
	if ( !OSVersion.empty() )
		return OSVersion;
	else
		return "Unknown OS";
}

string xStatsInfo::getOSEx(bool *dynamic)
{
	if ( !OSExtendedVersion.empty() )
		return OSExtendedVersion;
	else if ( !OSVersion.empty() )
		return OSVersion;
	else
		return "Unknown OS";
}

string xStatsInfo::getDateCreated(const vector<string> &arguments, bool *dynamic)
{
	if (arguments.size() < 1)
		return "[Error]";

	WIN32_FIND_DATA fd;
	HANDLE search = FindFirstFile(arguments[0].c_str(), &fd);
	FILETIME ftLocal;
	SYSTEMTIME st;

	if (search)
	{
		FindClose(search);
		FileTimeToLocalFileTime(&fd.ftCreationTime, &ftLocal);
		FileTimeToSystemTime(&ftLocal, &st);
		
		return formatDateTime((arguments.size() > 1) ? arguments[1] : DEFAULT_FILE_DATE_FORMAT, st, 0);
	}

	return "[Could Not Open File!]";
}

string xStatsInfo::getDateLastModified(const vector<string> &arguments, bool *dynamic)
{
	if (arguments.size() < 1)
		return "[Error]";

	WIN32_FIND_DATA fd;
	HANDLE search = FindFirstFile(arguments[0].c_str(), &fd);
	FILETIME ftLocal;
	SYSTEMTIME st;

	if (search)
	{
		FindClose(search);
		FileTimeToLocalFileTime(&fd.ftLastWriteTime, &ftLocal);
		FileTimeToSystemTime(&ftLocal, &st);
		
		return formatDateTime((arguments.size() > 1) ? arguments[1] : DEFAULT_FILE_DATE_FORMAT, st, 0);
	}

	return "[Could Not Open File!]";
}

string xStatsInfo::getFileExists(const string &file, bool *dynamic)
{
	WIN32_FIND_DATA fd;
	HANDLE search = FindFirstFile(file.c_str(), &fd);

	if (INVALID_HANDLE_VALUE != search)
	{
		FindClose(search);
		return "1";
	}
	
	FindClose(search);

	return "0";
}

string xStatsInfo::getFirstLine(const vector<string> &arguments, bool *dynamic)
{
	string file = arguments[0];

	while ( strstr(file.c_str(), "<br>") != NULL )
		file.replace(file.find("<br>"), 4, "\\n", 2);

	FILE *fp = fopen(file.c_str(), "r");
	char lineBuffer[MAX_LINE_LENGTH] = {0};

	if (fp)
	{
		if ( arguments.size() == 1 )
		{
			if (fgets(lineBuffer, MAX_LINE_LENGTH, fp))
			{
				int length = strlen(lineBuffer);
				if (lineBuffer[length - 1] == '\n') lineBuffer[length - 1] = 0;
				fclose(fp);
				return (string) lineBuffer;
			}
		}
		else if ( arguments.size() >= 2 )
		{
			int counter = 0;
			int amount = atoi( arguments[1].c_str() );
			string result = "";

			while ( counter < amount )
			{
				if (fgets(lineBuffer, MAX_LINE_LENGTH, fp))
				{
					int length = strlen(lineBuffer);

					if ( lineBuffer[length - 1] == '\n' )
						lineBuffer[length - 1] = 0;

					result.append( lineBuffer );
					counter++;
					if ( counter != amount )
						result.append( "<br>" );
				}
				else
				{
					counter = amount;
				}
			}

			fclose(fp);
			return result;
		}

		fclose(fp);
		return "";
	}

	return "[Could Not Open File!]";
}

string xStatsInfo::getLastLine(const vector<string> &arguments, bool *dynamic)
{
	string file = arguments[0];
	while ( strstr(file.c_str(), "<br>") != NULL )
		file.replace(file.find("<br>"), 4, "\\n", 2);

	FILE *fp = fopen(file.c_str(), "r");
	char lineBuffer[MAX_LINE_LENGTH] = {0};

	if(fp)
	{
		if ( arguments.size() == 1 )
		{
			strcpy(lineBuffer, "");

			while(fgets(lineBuffer, MAX_LINE_LENGTH, fp))
			{
				// Do nothing
			}

			int length = strlen(lineBuffer);
			if(lineBuffer[length - 1] == '\n')
				lineBuffer[length - 1] = 0;
			fclose(fp);
			return string(lineBuffer);
		}
		else if ( arguments.size() >= 2 )
		{
			int linecount = 0;
			while(fgets(lineBuffer, MAX_LINE_LENGTH, fp))
			{
				linecount++;
			}
			fclose(fp);

			string file = arguments[0];

			while ( strstr(file.c_str(), "<br>") != NULL )
				file.replace(file.find("<br>"), 4, "\\n", 2);

			fp = fopen(file.c_str(), "r");
			int counter = 0;
			int amount = atoi( arguments[1].c_str() );
			string result = "";

			while ( counter < linecount-amount )
			{
				if(fgets(lineBuffer, MAX_LINE_LENGTH, fp))
				{
					counter++;
				}
			}
			counter = 0;

			while ( counter < amount )
			{
				if(fgets(lineBuffer, MAX_LINE_LENGTH, fp))
				{
					int length = strlen(lineBuffer);

					if ( lineBuffer[length - 1] == '\n' )
						lineBuffer[length - 1] = 0;

					result.append( lineBuffer );
					counter++;
					if ( counter != amount )
						result.append( "<br>" );
				}
				else
				{
					counter = amount;
				}
			}

			fclose(fp);
			return result;
		}
	}

	return "[Could Not Open File!]";
}

string xStatsInfo::getLine(const vector<string> &arguments, bool *dynamic)
{
	if (arguments.size() < 2)
		return "[Error]";

	string result = "";
	string file = arguments[0];
	int lineNumberStart = atoi(arguments[1].c_str());
	int lineNumberEnd = lineNumberStart+1;
	if ( arguments.size() == 3 )
		lineNumberEnd = atoi(arguments[2].c_str());

	while ( strstr(file.c_str(), "<br>") != NULL )
		file.replace(file.find("<br>"), 4, "\\n", 2);

	FILE *fp = fopen(file.c_str(), "r");
	int lineCount = 0;
	char lineBuffer[MAX_LINE_LENGTH] = {0};

	if (fp)
	{
		while (fgets(lineBuffer, MAX_LINE_LENGTH, fp))
		{	
			if ( arguments.size() == 2 )
			{
				if (lineCount == lineNumberStart-1)
				{
					int length = strlen(lineBuffer);
					if (lineBuffer[length - 1] == '\n')
						lineBuffer[length - 1] = 0;
					fclose(fp);
					return string(lineBuffer);
				}
			}
			else
			{
				if (lineCount >= lineNumberStart-1 && lineCount <= lineNumberEnd-1)
				{
					int length = strlen(lineBuffer);
					if (lineBuffer[length - 1] == '\n')
						lineBuffer[length - 1] = 0;
						
					result.append(lineBuffer);
					result.append("<br>");
				}
			}

			lineCount++;
		}

		fclose(fp);
		if ( arguments.size() == 2 )
			return "";
		else
			return result;
	}

	return "[Could Not Open File!]";
}

string xStatsInfo::getMatchLine(const vector<string> &arguments, bool *dynamic)
{
	if (arguments.size() < 2)
		return "[Error]";

	string file = arguments[0];
	string target = arguments[1];

	while ( strstr(file.c_str(), "<br>") != NULL )
		file.replace(file.find("<br>"), 4, "\\n", 2);

	FILE *fp = fopen(file.c_str(), "r");
	int lineCount = 0;
	char lineBuffer[MAX_LINE_LENGTH] = {0};

	if (fp)
	{
		while (fgets(lineBuffer, MAX_LINE_LENGTH, fp))
		{
			int length = strlen(lineBuffer);
			if (lineBuffer[length - 1] == '\n') lineBuffer[length - 1] = 0;

			if ( match( target.c_str(), lineBuffer ) )
			{
				fclose(fp);
				return string(lineBuffer);
			}

			lineCount++;
		}

		fclose(fp);
		return "";
	}

	return "[Could Not Open File!]";
}

string xStatsInfo::getLineCount(const string &file, bool *dynamic)
{
	string process = file;

	while ( strstr(process.c_str(), "<br>") != NULL )
		process.replace(process.find("<br>"), 4, "\\n", 2);

	FILE *fp = fopen(process.c_str(), "r");
	int lineCount = 0;
	char lineBuffer[MAX_LINE_LENGTH] = {0};

	if (fp)
	{
		while (fgets(lineBuffer, MAX_LINE_LENGTH, fp))
		{
			lineCount++;
		}

		fclose(fp);
		sprintf(lineBuffer, "%d", lineCount);
		return (string) lineBuffer;
	}

	return "[Could Not Open File!]";
}

string xStatsInfo::getRandomLine(const string &file, bool *dynamic)
{
	string process = file;

	while ( strstr(process.c_str(), "<br>") != NULL )
		process.replace(process.find("<br>"), 4, "\\n", 2);

	FILE *fp = fopen(process.c_str(), "r");
	int lineCount = 0;
	char lineBuffer[MAX_LINE_LENGTH] = {0};

	if (fp)
	{
		while (fgets(lineBuffer, MAX_LINE_LENGTH, fp))
		{
			if (lineBuffer[0] != '\n' && lineBuffer[0] != 0)
				lineCount++;
		}

		srand(GetTickCount());
		int line = rand() % lineCount;

		lineCount = 0;
		fseek(fp, 0, SEEK_SET);

		while (fgets(lineBuffer, MAX_LINE_LENGTH, fp))
		{
			if (lineBuffer[0] != '\n' && lineBuffer[0] != 0)
			{
				if (lineCount == line)
				{
					int length = strlen(lineBuffer);
					if(lineBuffer[length - 1] == '\n') lineBuffer[length - 1] = 0;
					fclose(fp);
					return (string) lineBuffer;
				}

				lineCount++;
			}
		}

		fclose(fp);
		return "[Error]";
	}

	return "[Could Not Open File!]";
}

string xStatsInfo::getSize(const vector<string> &arguments, bool *dynamic)
{
	if (arguments.size() < 1)
		return "[error]";

	int units = unitsDefault;

	if (arguments.size() >= 2)
		units = atoi(arguments[1].c_str());

	WIN32_FIND_DATA fd;
	HANDLE search = FindFirstFile(arguments[0].c_str(), &fd);

	if (search)
	{
		FindClose(search);
		largeInt size = (((largeInt) fd.nFileSizeHigh) << 32) | ((largeInt) fd.nFileSizeLow);
		return formatByteSize(size, 0, units);
	}

	return "[Could Not Open File!]";
}

string xStatsInfo::getSwapAvailable(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;
	
	int units = unitsDefault;

	if (arguments.size() >= 1)
		units = atoi(arguments[0].c_str());

	if ( IsOS(OS_2KXP) )
	{
		MEMORYSTATUSEX ms;
		ms.dwLength = sizeof(MEMORYSTATUSEX);
		GlobalMemoryStatusEx(&ms);

		return formatByteSize(ms.ullAvailPageFile, ms.ullTotalPageFile, units);
	}
	else
	{
		MEMORYSTATUS ms;
		ms.dwLength = sizeof(MEMORYSTATUS);
		GlobalMemoryStatus(&ms);

		return formatByteSize(ms.dwAvailPageFile, ms.dwTotalPageFile, units);
	}
}

string xStatsInfo::getSwapInUse(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;
	
	int units = unitsDefault;

	if (arguments.size() >= 1)
		units = atoi(arguments[0].c_str());

	if ( IsOS(OS_2KXP) )
	{
		MEMORYSTATUSEX ms;
		ms.dwLength = sizeof(MEMORYSTATUSEX);
		GlobalMemoryStatusEx(&ms);

		return formatByteSize(ms.ullTotalPageFile - ms.ullAvailPageFile, ms.ullTotalPageFile, units);
	}
	else
	{
		MEMORYSTATUS ms;
		ms.dwLength = sizeof(MEMORYSTATUS);
		GlobalMemoryStatus(&ms);

		return formatByteSize(ms.dwTotalPageFile - ms.dwAvailPageFile, ms.dwTotalPageFile, units);
	}
}

string xStatsInfo::getSwapTotal(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	int units = unitsDefault;

	if (arguments.size() >= 1)
		units = atoi(arguments[0].c_str());

	if ( IsOS(OS_2KXP) )
	{
		MEMORYSTATUSEX ms;
		ms.dwLength = sizeof(MEMORYSTATUSEX);
		GlobalMemoryStatusEx(&ms);

		return formatByteSize(ms.ullTotalPageFile, ms.ullTotalPageFile, units);
	}
	else
	{
		MEMORYSTATUS ms;
		ms.dwLength = sizeof(MEMORYSTATUS);
		GlobalMemoryStatus(&ms);

		return formatByteSize(ms.dwTotalPageFile, ms.dwTotalPageFile, units);
	}
}

string xStatsInfo::getTime(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	SYSTEMTIME st;
	getLocalizedTime(&st, arguments.size() >= 2 ? arguments[1] : "");

	if(arguments.size() >= 1)
	{
		return formatDateTime(arguments[0], st, 0);
	}
	else
	{
		char buffer[64] = {0};
		GetTimeFormat(LOCALE_USER_DEFAULT, TIME_NOSECONDS, &st, 0, buffer, 64);
		return string(buffer);
	}
}

string xStatsInfo::getUptime(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	unsigned int ms = GetTickCount();
	unsigned int seconds = ms / 1000;
	ms %= 1000;
	unsigned int minutes = seconds / 60;
	seconds %= 60;
	unsigned int hours = minutes / 60;
	minutes %= 60;
	unsigned int days = hours / 24;
	hours %= 24;

	SYSTEMTIME st;

	st.wMonth = -1;
	st.wDay = days;
	st.wDayOfWeek = -1;
	st.wYear = 0;
	st.wHour = hours;
	st.wMinute = minutes;
	st.wSecond = seconds;
	st.wMilliseconds = ms;

	string format = (arguments.size() >= 1) ? arguments[0] : "d days hh:nn:ss";
	return formatDateTime(format, st, 1);
}

string xStatsInfo::getUserName(bool *dynamic)
{
	char buffer[32] = {0};
	unsigned long bufferLen = 32;

	GetUserName(buffer, &bufferLen);

	return (string) buffer;
}

string xStatsInfo::getWinampSong(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

	if (hwndWA2 != NULL)
	{
		char buffer[256] = {0};
		GetWindowText(hwndWA2, buffer, 256);

		if ( ((string) buffer).empty() )
		{
			if (arguments.size() != 0)
				return arguments[0];
			else
				return "";
		}
		else
		{
			return trim(between(string(buffer), ".", "-"));
		}
	}

	if (arguments.size() != 0)
		return arguments[0];
	else
		return "";
}

string xStatsInfo::getWinampArtist(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

	if(hwndWA2 != NULL)
	{
		char buffer[256] = {0};
		GetWindowText(hwndWA2, buffer, 256);

		if ( ((string) buffer).empty() )
		{
			if (arguments.size() != 0)
				return arguments[0];
			else
				return "";
		}
		else
		{
			return trim(before(between(string(buffer), ".", "-"), "-"));
		}
	}

	if (arguments.size() != 0)
		return arguments[0];
	else
		return "";
}

string xStatsInfo::getWinampTitle(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

	if(hwndWA2 != NULL)
	{
		char buffer[256] = {0};
		GetWindowText(hwndWA2, buffer, 256);

		if ( ((string) buffer).empty() )
		{
			if (arguments.size() != 0)
				return arguments[0];
			else
				return "";
		}
		else
		{
			return trim(after(between(string(buffer), ".", "-"), "-"));
		}
	}

	if (arguments.size() != 0)
		return arguments[0];
	else
		return "";
}

string xStatsInfo::getWinampTime(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	int milliseconds = 0;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);
	
	if (hwndWA2)
		milliseconds = SendMessage(hwndWA2, WM_USER, 0, 105);

	char buffer[80] = {0};

	int seconds = milliseconds / 1000;
	if ( arguments.size() == 0 )
	{
		int minutes = seconds / 60;
		seconds = seconds % 60;	
	
		sprintf(buffer, "%d:%02d", minutes, seconds);
	}
	else if ( arguments.size() != 0 )
	{
		if ( _stricmp(arguments[0].c_str(), "sec") == NULL )
			sprintf(buffer, "%d", seconds);
		else
		{
			int minutes = seconds / 60;
			seconds = seconds % 60;	
		
			sprintf(buffer, "%d:%02d", minutes, seconds);
		}
	}

	if (milliseconds != 0)
		return string(buffer);
	else
		return "";
}

string xStatsInfo::getWinampRemainTime(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	int milliseconds = 0;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);
	
	if(hwndWA2)
		milliseconds = (SendMessage(hwndWA2, WM_USER, 1, 105) * 1000) - SendMessage(hwndWA2, WM_USER, 0, 105);

	char buffer[80] = {0};

	int seconds = milliseconds / 1000;
	if (arguments.size() == 0)
	{
		int minutes = seconds / 60;
		seconds = seconds % 60;	
	
		sprintf(buffer, "%d:%02d", minutes, seconds);
	}
	else if ( arguments.size() != 0 )
	{
		if ( _stricmp(arguments[0].c_str(), "sec") == NULL )
			sprintf(buffer, "%d", seconds);
		else
		{
			int minutes = seconds / 60;
			seconds = seconds % 60;	
		
			sprintf(buffer, "%d:%02d", minutes, seconds);
		}
	}

	if (milliseconds != 0)
		return string(buffer);
	else
		return "";
}

string xStatsInfo::getWinampTotalTime(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	int milliseconds = 0;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);
	
	if(hwndWA2)
		milliseconds = SendMessage(hwndWA2, WM_USER, 1, 105) * 1000;

	char buffer[80] = {0};

	int seconds = milliseconds / 1000;
	if (arguments.size() == 0)
	{
		int minutes = seconds / 60;
		seconds = seconds % 60;	
	
		sprintf(buffer, "%d:%02d", minutes, seconds);
	}
	else if ( arguments.size() != 0 )
	{
		if ( _stricmp(arguments[0].c_str(), "sec") == NULL )
			sprintf(buffer, "%d", seconds);
		else
		{
			int minutes = seconds / 60;
			seconds = seconds % 60;	
		
			sprintf(buffer, "%d:%02d", minutes, seconds);
		}
	}

	if (milliseconds != 0)
		return string(buffer);
	else
		return "";
}

string xStatsInfo::getWinampStatus(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

	if (hwndWA2)
	{
		int status = SendMessage(hwndWA2, WM_USER, 0, 104);

		switch(status)
		{
		case 1:
			return "Playing";

		case 3:
			return "Paused";

		default:
			return "Stopped";
		}
	}

	if (arguments.size() != 0)
		return arguments[0];
	else
		return "";
}

string xStatsInfo::getWinampBitRate(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

	if (hwndWA2)
		return (IntToString(SendMessage(hwndWA2, WM_USER, 1, 126)) + "kbit");
	else
		return "";
}

string xStatsInfo::getWinampSampleRate(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

	if (hwndWA2)
	{
		int hz = SendMessage(hwndWA2, WM_USER, 0, 126);
		if (hz > 1000)
			hz = hz/1000;

		return (IntToString(hz) + "kHz");
	}
	else
		return "";
}

string xStatsInfo::getWinampPlaying(bool *dynamic)
{
	if (dynamic) *dynamic = true;
	
	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);
	
	string retval = "0";

	if (hwndWA2)
	{
		if (SendMessage(hwndWA2, WM_USER, 0, 104) == 1)
			retval = "1";
	}

	return retval;
}

string xStatsInfo::getWinampPaused(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);
	
	string retval = "0";

	if (hwndWA2)
	{
		if (SendMessage(hwndWA2, WM_USER, 0, 104) == 3)
			retval = "1";
	}

	return retval;
}

string xStatsInfo::getWinampStopped(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

	string retval = "0";

	if (hwndWA2)
	{
		int code = SendMessage(hwndWA2, WM_USER, 0, 104);
		if (code != 1 && code != 3)
			retval = "1";
	}

	return retval;
}

string xStatsInfo::getWinampRepeat(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

	if (hwndWA2)
		return (IntToString( SendMessage(hwndWA2, WM_USER, 0, 251) ));
	else 
		return "0";
}

string xStatsInfo::getWinampShuffle(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);

	if (hwndWA2)
		return (IntToString( SendMessage(hwndWA2,WM_USER,0,250) ));
	else
		return "0";;
}

// .alur
string xStatsInfo::getExtendedWinampFileInfo(char* file, const char* field, HANDLE hWinamp, HWND hwndWA2)
{
  char rField[64];
  char metadata[256] = {0};
  extendedFileInfoStruct exFIS = {0};
 
  // We need field to be a char
  strcpy(rField, field);
 
  // Allocate some memory in winamps address space
  char *remBuf = (char *)VirtualAllocEx(hWinamp, NULL, sizeof(metadata), MEM_COMMIT, PAGE_READWRITE);
  char *remMetaDataBuf = (char *)VirtualAllocEx(hWinamp, NULL, sizeof(rField), MEM_COMMIT, PAGE_READWRITE);
  char *remStruct = (char *)VirtualAllocEx(hWinamp, NULL, sizeof(exFIS), MEM_COMMIT, PAGE_READWRITE);
  char *remFileBuf = (char *)VirtualAllocEx(hWinamp, NULL, 512, MEM_COMMIT, PAGE_READWRITE);
 
  // Fill in the data for the extendedfileinfostruct
  exFIS.filename = remFileBuf;
  exFIS.metadata = remMetaDataBuf;
  exFIS.ret = remBuf;
  exFIS.retlen = sizeof(metadata);
 
  // Copy over the extendedfileinfostruct and it's data to winamps address space
  WriteProcessMemory(hWinamp, remFileBuf, file, 512, NULL);
  WriteProcessMemory(hWinamp, remMetaDataBuf, rField, sizeof(rField), NULL);
  WriteProcessMemory(hWinamp, remStruct, &exFIS, sizeof(exFIS), NULL);
 
  // Have winamp read the extendedfileinfostruct and put the desired metadata in remBuf
  if (!SendMessage(hwndWA2, WM_USER, (WPARAM)remStruct, 296))
    return "";
 
  // Copy remBuf to local memory
  ReadProcessMemory(hWinamp, remBuf, &metadata, sizeof(metadata), NULL);
 
  // Free up memory allocated in winamps address space
  VirtualFreeEx(hWinamp, remBuf, sizeof(metadata), MEM_DECOMMIT);
  VirtualFreeEx(hWinamp, remBuf, 0, MEM_RELEASE);
  VirtualFreeEx(hWinamp, remMetaDataBuf, sizeof(rField), MEM_DECOMMIT);
  VirtualFreeEx(hWinamp, remMetaDataBuf, 0, MEM_RELEASE);
  VirtualFreeEx(hWinamp, remStruct, sizeof(exFIS), MEM_DECOMMIT);
  VirtualFreeEx(hWinamp, remStruct, 0, MEM_RELEASE);
  VirtualFreeEx(hWinamp, remFileBuf, sizeof(file), MEM_DECOMMIT);
  VirtualFreeEx(hWinamp, remFileBuf, 0, MEM_RELEASE);
 
  return string(metadata);
}
// .alur

// .alur
string xStatsInfo::getWinampInfo(int iWhichInfo, bool *dynamic)
{
  if (dynamic) *dynamic = true;
 
  // Get winamps hwnd
  HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);
  if (hwndWA2 == NULL)
    return "";
 
  // Get the currently playing track.
  int TempTrack = SendMessage(hwndWA2, WM_USER, 0, 125);
 
  // Only get new information if the currenly playing track has changed.
  if (TempTrack != winampCurrentTrack)
  {
    char szPlayingFile[512];
    HANDLE hWinamp;
    unsigned long dWinamp;
 
    // Get winamps process id
    GetWindowThreadProcessId(hwndWA2, &dWinamp);
 
    // Open a handle to winamp
    hWinamp = OpenProcess(PROCESS_ALL_ACCESS, false, dWinamp);
    if (hWinamp != NULL)
    {
		// Get a pointer to the currently playing item.
		void* winampTrack = (void *)SendMessage(hwndWA2, WM_USER, TempTrack, 211);
 
		// Copy the string over to local memory.
		ReadProcessMemory(hWinamp, winampTrack, &szPlayingFile, sizeof(szPlayingFile), NULL);
 
		// Get information about the playing item.
		winampAlbum = getExtendedWinampFileInfo(szPlayingFile, "Album", hWinamp, hwndWA2);
		winampGenre = getExtendedWinampFileInfo(szPlayingFile, "Genre", hWinamp, hwndWA2);
		winampAlbumTrack = getExtendedWinampFileInfo(szPlayingFile, "Track", hWinamp, hwndWA2);
		winampYear = getExtendedWinampFileInfo(szPlayingFile, "Year", hWinamp, hwndWA2);
		winampComment = getExtendedWinampFileInfo(szPlayingFile, "Comment", hWinamp, hwndWA2);
 
		// Close the handle to winamp
		CloseHandle(hWinamp);
    }
 
    // Prevent this code from being called again until the song has changed.
    winampCurrentTrack = TempTrack;
  }
 
  // Return the correct string.
  switch (iWhichInfo)
  {
    case 0:
		return winampAlbum;
    case 1:
		return winampGenre;
    case 2:
		return winampAlbumTrack;
    case 3:
		return winampYear;
    case 4:
		return winampComment;
  }

  return "";
}
// .alur

// .alur
string xStatsInfo::getWinampRating(bool *dynamic)
{
  if (dynamic) *dynamic = true;
 
  HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);
 
  char buffer[80] = {0};
 
  int rating = 0;
 
  if (hwndWA2 != NULL)
    rating = SendMessage(hwndWA2, WM_USER, 0, 640);
 
  itoa(rating, buffer, 10);
 
  return string(buffer);
}
// .alur

string xStatsInfo::getWindowTitle(const string &windowClass, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	HWND hWnd = FindWindow(windowClass.c_str(), 0);
	if (!hWnd) 
		return "";

	char buffer[256] = {0};
	GetWindowText(hWnd, buffer, 256);

	return (string)buffer;
}

string xStatsInfo::getPowerSource(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	SYSTEM_POWER_STATUS sps;
	GetSystemPowerStatus(&sps);

	if (sps.ACLineStatus == 0)
	{
		if (arguments.size() >= 2)
			return arguments[1];
		else
			return "Battery";
	}
	else
	{
		if (arguments.size() >= 1)
			return arguments[0];
		else
			return "AC";
	}
}

string xStatsInfo::getBattery(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	SYSTEM_POWER_STATUS sps;
	GetSystemPowerStatus(&sps);

	if (sps.BatteryLifePercent > 100)
		sps.BatteryLifePercent = 0;

	char output[8] = {0};
	StringCchPrintf(output, 8, "%d%%", sps.BatteryLifePercent);

	return (string) output;
}

// -----------------------------------
//  MotherBoard Monitor 5 (MBM5) info
// -----------------------------------
/*
sensortype:		the type of sensor it should return (temp, voltage, fan, etc)
arguments:		# of sensor (you can have more than one temp sensor)
dynamic:		if this is a dynamic variable which can change over time
bUseFahrenheit:	if we should return the values in Fahrenheit
*/
string xStatsInfo::getMBMData (const SensorType sensortype, const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;	//this is always a dynamic variable

	int n = 0;
	double value = 0;
	mbmSharedData *pSharedData = NULL;
	char output[16];

	if(arguments.size() >= 1)
		n = atoi(arguments[0].c_str()) - 1;

	if(n < 0) n = 0;		//n has now the # of the sensor were looking for

	HANDLE hSharedData=OpenFileMapping(FILE_MAP_READ, FALSE, "$M$B$M$5$S$D$");
	if (hSharedData==0)
		return "0";
	
	if(hSharedData != INVALID_HANDLE_VALUE)
	{	pSharedData = (mbmSharedData *) MapViewOfFile(hSharedData,
			FILE_MAP_READ, 0, 0, 0);
	if(!pSharedData) {
			CloseHandle (hSharedData);
			UnmapViewOfFile(pSharedData);
			return "0";
		}
	}

	//this only works with newer versions of MBM
	if (pSharedData->sdVersion < 51090) {
		return "Please update your version of MBM";
	}

	int totalSensorCount = 0;
	for (int i=0;i<5;i++)
		totalSensorCount +=	pSharedData->sdIndex[i].Count;


	for (int j=0;j<totalSensorCount;j++)
	{	
		if (pSharedData->sdSensor[j].ssType == sensortype)	//is this the type of sensor were looking for?
		{
			if (n == 0)		//have we found our sensor?
			{
				switch (sensortype) 
				{
					case stTemperature:

						value = pSharedData->sdSensor[j].ssCurrent;
						StringCbPrintf (output, 15, "%.0f", value);
						break;
					case stVoltage:
					case stFan:
						value = pSharedData->sdSensor[j].ssCurrent;
						StringCbPrintf (output, 15, "%.2f", value);
						break;
				}
				UnmapViewOfFile(pSharedData);
				CloseHandle (hSharedData);
				return string(output);
			}
			else		//no we havent found it yet: same type, but wrong number
			{
				n--;
			}
		}
	}

	UnmapViewOfFile(pSharedData);
	CloseHandle (hSharedData);
	return "0";
}

string xStatsInfo::getMBMLoaded(bool *dynamic)
{
	HANDLE memHandle = OpenFileMapping(FILE_MAP_READ, FALSE, "$M$B$M$5$S$D$");
	string retval = "0";

	if(memHandle)
	{
		retval = "1";
		CloseHandle(memHandle);
	}

	if(dynamic) *dynamic = true;
	return retval;
}

string xStatsInfo::getNetAdapterName(const vector<string> &arguments, bool *dynamic)
{
	if (!bNetStatsStarted)
	{
		NetStats *netStats = new NetStats(0);
		
		bNetStatsStarted = true;
		netStats->Start();
		
		netStatsList.insert(netStatsList.end(), netStats);
	}

	int SelectedInterface = 0;
	NetStats *usedNetStats = (*netStatsList.begin());

	if ( arguments.size() == 1 )
	{
		SelectedInterface = atoi(arguments[0].c_str());

		if ( SelectedInterface > 0 )
		{
			int newInterface = 1;
			while ( SelectedInterface > netStatsList.size()-1 )
			{
				NetStats *netStats = new NetStats(newInterface);
			
				netStats->Start();
				
				netStatsList.insert(netStatsList.end(), netStats);

				newInterface++;
			}

			int counter = 0;
			for(NetStatsListIterator nsit = netStatsList.begin(); nsit != netStatsList.end(); nsit++)
			{
				if ( SelectedInterface == counter )
				{
					usedNetStats = (*nsit);
					break;
				}
				counter++;
			}
		}
	}

	//if(dynamic) *dynamic = true;

	return (string) usedNetStats->GetMyInterfaceName();
}

string xStatsInfo::getNetIn(const vector<string> &arguments, bool *dynamic)
{
	if (!bNetStatsStarted)
	{
		NetStats *netStats = new NetStats(0);
		
		bNetStatsStarted = true;
		netStats->Start();
		
		netStatsList.insert(netStatsList.end(), netStats);
	}

	int SelectedInterface = 0;
	NetStats *usedNetStats = (*netStatsList.begin());

	if ( arguments.size() == 1 )
	{
		SelectedInterface = atoi(arguments[0].c_str());

		if ( SelectedInterface > 0 )
		{
			int newInterface = 1;
			while ( SelectedInterface > netStatsList.size()-1 )
			{
				NetStats *netStats = new NetStats(newInterface);
			
				netStats->Start();
				
				netStatsList.insert(netStatsList.end(), netStats);

				newInterface++;
			}

			int counter = 0;
			for(NetStatsListIterator nsit = netStatsList.begin(); nsit != netStatsList.end(); nsit++)
			{
				if ( SelectedInterface == counter )
				{
					usedNetStats = (*nsit);
					break;
				}
				counter++;
			}
		}
	}

	if(dynamic) *dynamic = true;

	return (string) usedNetStats->GetBytesPerSecIn();
}

string xStatsInfo::getNetOut(const vector<string> &arguments, bool *dynamic)
{
	if (!bNetStatsStarted)
	{
		NetStats *netStats = new NetStats(0);
		
		bNetStatsStarted = true;
		netStats->Start();
		
		netStatsList.insert(netStatsList.end(), netStats);
	}

	int SelectedInterface = 0;
	NetStats *usedNetStats = (*netStatsList.begin());

	if ( arguments.size() == 1 )
	{
		SelectedInterface = atoi(arguments[0].c_str());

		if ( SelectedInterface > 0 )
		{
			int newInterface = 1;
			while ( SelectedInterface > netStatsList.size()-1 )
			{
				NetStats *netStats = new NetStats(newInterface);
			
				netStats->Start();
				
				netStatsList.insert(netStatsList.end(), netStats);

				newInterface++;
			}

			int counter = 0;
			for(NetStatsListIterator nsit = netStatsList.begin(); nsit != netStatsList.end(); nsit++)
			{
				if ( SelectedInterface == counter )
				{
					usedNetStats = (*nsit);
					break;
				}
				counter++;
			}
		}
	}

	if(dynamic) *dynamic = true;

	return (string) usedNetStats->GetBytesPerSecOut();
}

string xStatsInfo::getNetInOut(const vector<string> &arguments, bool *dynamic)
{
	if (!bNetStatsStarted)
	{
		NetStats *netStats = new NetStats(0);
		
		bNetStatsStarted = true;
		netStats->Start();
		
		netStatsList.insert(netStatsList.end(), netStats);
	}

	int SelectedInterface = 0;
	NetStats *usedNetStats = (*netStatsList.begin());

	if ( arguments.size() == 1 )
	{
		SelectedInterface = atoi(arguments[0].c_str());

		if ( SelectedInterface > 0 )
		{
			int newInterface = 1;
			while ( SelectedInterface > netStatsList.size()-1 )
			{
				NetStats *netStats = new NetStats(newInterface);
			
				netStats->Start();
				
				netStatsList.insert(netStatsList.end(), netStats);

				newInterface++;
			}

			int counter = 0;
			for(NetStatsListIterator nsit = netStatsList.begin(); nsit != netStatsList.end(); nsit++)
			{
				if ( SelectedInterface == counter )
				{
					usedNetStats = (*nsit);
					break;
				}
				counter++;
			}
		}
	}

	if(dynamic) *dynamic = true;

	return (string) usedNetStats->GetTotalBytesPerSec();
}

string xStatsInfo::getNetTotalIn(const vector<string> &arguments, bool *dynamic)
{
	if (!bNetStatsStarted)
	{
		NetStats *netStats = new NetStats(0);
		
		bNetStatsStarted = true;
		netStats->Start();
		
		netStatsList.insert(netStatsList.end(), netStats);
	}

	int SelectedInterface = 0;
	NetStats *usedNetStats = (*netStatsList.begin());

	if ( arguments.size() == 1 )
	{
		SelectedInterface = atoi(arguments[0].c_str());

		if ( SelectedInterface > 0 )
		{
			int newInterface = 1;
			while ( SelectedInterface > netStatsList.size()-1 )
			{
				NetStats *netStats = new NetStats(newInterface);
			
				netStats->Start();
				
				netStatsList.insert(netStatsList.end(), netStats);

				newInterface++;
			}

			int counter = 0;
			for(NetStatsListIterator nsit = netStatsList.begin(); nsit != netStatsList.end(); nsit++)
			{
				if ( SelectedInterface == counter )
				{
					usedNetStats = (*nsit);
					break;
				}
				counter++;
			}
		}
	}

	if(dynamic) *dynamic = true;

	return (string) usedNetStats->GetAllBytesIn();
}

string xStatsInfo::getNetTotalOut(const vector<string> &arguments, bool *dynamic)
{
	if (!bNetStatsStarted)
	{
		NetStats *netStats = new NetStats(0);
		
		bNetStatsStarted = true;
		netStats->Start();
		
		netStatsList.insert(netStatsList.end(), netStats);
	}

	int SelectedInterface = 0;
	NetStats *usedNetStats = (*netStatsList.begin());

	if ( arguments.size() == 1 )
	{
		SelectedInterface = atoi(arguments[0].c_str());

		if ( SelectedInterface > 0 )
		{
			int newInterface = 1;
			while ( SelectedInterface > netStatsList.size()-1 )
			{
				NetStats *netStats = new NetStats(newInterface);
			
				netStats->Start();
				
				netStatsList.insert(netStatsList.end(), netStats);

				newInterface++;
			}

			int counter = 0;
			for(NetStatsListIterator nsit = netStatsList.begin(); nsit != netStatsList.end(); nsit++)
			{
				if ( SelectedInterface == counter )
				{
					usedNetStats = (*nsit);
					break;
				}
				counter++;
			}
		}
	}

	if(dynamic) *dynamic = true;

	return (string) usedNetStats->GetAllBytesOut();
}

string xStatsInfo::getNetTotalInOut(const vector<string> &arguments, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	if (!bNetStatsStarted)
	{
		NetStats *netStats = new NetStats(0);
		
		bNetStatsStarted = true;
		netStats->Start();
		
		netStatsList.insert(netStatsList.end(), netStats);
	}

	int SelectedInterface = 0;
	NetStats *usedNetStats = (*netStatsList.begin());

	if ( arguments.size() == 1 )
	{
		SelectedInterface = atoi(arguments[0].c_str());

		if ( SelectedInterface > 0 )
		{
			int newInterface = 1;
			while ( SelectedInterface > netStatsList.size()-1 )
			{
				NetStats *netStats = new NetStats(newInterface);
			
				netStats->Start();
				
				netStatsList.insert(netStatsList.end(), netStats);

				newInterface++;
			}

			int counter = 0;
			for(NetStatsListIterator nsit = netStatsList.begin(); nsit != netStatsList.end(); nsit++)
			{
				if ( SelectedInterface == counter )
				{
					usedNetStats = (*nsit);
					break;
				}
				counter++;
			}
		}
	}

	return (string) usedNetStats->GetAllTotalBytes();
}

string xStatsInfo::exportedEvars(const string &s, bool *dynamic)
{
	if (dynamic) *dynamic = true;

	return iGetxRCLine(s, "", ("Evar: " + s + " Exists Not"));
}

string xStatsInfo::empty(const string &s, bool *dynamic)
{
	return s.empty() ? "1" : "0";
}

string xStatsInfo::notEmpty(const string &s, bool *dynamic)
{
	return s.empty() ? "0" : "1";
}

string xStatsInfo::after(const string &s, const string &search)
{
	int i = s.find(search);
	i = (i < 0) ? 0 : i + search.length();
	return s.substr(i, s.length() - i);
}

string xStatsInfo::afterLast(const string &s, const string &search)
{
	int i = s.rfind(search);
	i = (i < 0) ? 0 : i + search.length();
	return s.substr(i, s.length() - i);
}

string xStatsInfo::before(const string &s, const string &search)
{
	int i = s.find(search);
	i = (i < 0) ? s.length() : i;
	return s.substr(0, i);
}

string xStatsInfo::beforeLast(const string &s, const string &search)
{
	int i = s.rfind(search);
	i = (i < 0) ? s.length() : i;
	return s.substr(0, i);
}

string xStatsInfo::between(const string &s, const string &left, const string &right, const string &modifier)
{
	int start = -1;
	int end = -1;
	if ( _stricmp(modifier.c_str(), "index") == NULL )
	{
		start = atoi( left.c_str() )-1;
		end = atoi( right.c_str() )-1;
	}
	else if ( _stricmp(modifier.c_str(), "indexspan") == NULL )
	{
		start = atoi( left.c_str() )-1;
		end = start + atoi( right.c_str() )-1;
	}

	if ( start < 0 || start > s.length() )
	{
		start = s.find(left);

		start = (start < 0) ? 0 : start + left.length();
	}

	if ( end < 0 || end > s.length() )
	{
		end = s.rfind(right);

		end = (end < 0) ? s.length() - 1 : end - right.length();
	}

	return (start > end) ? "" : s.substr(start, end - start + 1);
}

string xStatsInfo::capitalize(const string &s)
{
	string result = "";
	
	int length = s.length();
	int i = 0;
	
	while(i < length)
		result.append(1, (i == 0) ? toupper(s[i++]) : tolower(s[i++]));
	
	return result;
}

string xStatsInfo::lowerCase(const string &s)
{
	string result = "";
	
	int length = s.length();
	int i = 0;
	
	while(i < length)
		result.append(1, tolower(s[i++]));
	
	return result;
}

string xStatsInfo::trim(const string &s, const string &search)
{
	int start = 0;
	int end = s.length() - 1;
	
	while(start <= end && (strcspn( &s[start], " " ) == NULL || strcspn( &s[start], "\t" ) == NULL || strcspn( &s[start], search.c_str() ) == NULL))
		start++;
	
	while(start <= end && (strcspn( &s[start], " " ) == NULL || strcspn( &s[start], "\t" ) == NULL || strcspn( &s[end], search.c_str() ) == NULL))
		end--;
	
	return s.substr(start, end - start + 1);
}

string xStatsInfo::remove(const string &s, const string &search)
{
	CString worker = s.c_str();
	int start = 0;
	int end = search.length() - 1;

	while(start <= end)
	{
		worker.Remove( search[start] );
		start++;
	}
	
	return (string) worker;
}

string xStatsInfo::replace(const string &s, const string &search, const string &replace)
{
	CString worker = s.c_str();

	worker.Replace(search.c_str(), replace.c_str());
	
	return (string) worker;
}

string xStatsInfo::upperCase(const string &s)
{
	string result = "";
	
	int length = s.length();
	int i = 0;
	
	while(i < length)
		result.append(1, toupper(s[i++]));
	
	return result;
}

string xStatsInfo::verticaldown(const string &s)
{
	string result = "";
	
	int length = s.length();
	int i = 0;
	
	while(i < length)
	{
		result.append(1, s[i]);
		result.append("<br>");
		i++;
	}
	
	return result;
}

string xStatsInfo::verticalup(const string &s)
{
	string result = "";
	
	int i = s.length()-1;
	
	while(i >= 0)
	{
		result.append(1, s[i]);
		result.append("<br>");
		i--;
	}
	
	return result;
}

void xStatsInfo::getDiskFreeSpace(const string &drive, largeInt &freeBytes, largeInt &totalBytes)
{
	static BOOL (WINAPI *pfnGetDiskFreeSpaceEx)(LPCSTR, PULARGE_INTEGER, PULARGE_INTEGER, PULARGE_INTEGER);
	static BOOL fChecked = FALSE;

	if (!fChecked)
	{
		pfnGetDiskFreeSpaceEx = (BOOL (WINAPI *)(LPCSTR, PULARGE_INTEGER, PULARGE_INTEGER, PULARGE_INTEGER)) GetProcAddress(
			GetModuleHandle("KERNEL32.DLL"), "GetDiskFreeSpaceExA");

		fChecked = TRUE;
	}

	if (pfnGetDiskFreeSpaceEx)
	{
		largeInt dummy;

		pfnGetDiskFreeSpaceEx(drive.c_str(),
			(PULARGE_INTEGER) &freeBytes,
			(PULARGE_INTEGER) &totalBytes,
			(PULARGE_INTEGER) &dummy);
	}
	else
	{
		DWORD sectorsPerCluster;
		DWORD bytesPerSector;
		DWORD freeClusters;
		DWORD totalClusters;

		GetDiskFreeSpace(drive.c_str(),
			&sectorsPerCluster,
			&bytesPerSector,
			&freeClusters,
			&totalClusters);

		largeInt bytesPerCluster = (largeInt) bytesPerSector * (largeInt) sectorsPerCluster;

		freeBytes = (largeInt) freeClusters * bytesPerCluster;
		totalBytes = (largeInt) totalClusters * bytesPerCluster;
	}
}

static const char *units[] = {
	"bytes", "KB", "MB", "GB", "TB", "PB", "EB"
};

string xStatsInfo::formatByteSize(largeInt byteSize, largeInt total, int unit)
{
	char buffer[32] = {0};

	if (unit == unitsPercent)
	{
		if (total == 0)
		{
			return "0%";
		}
		else
		{
			StringCchPrintf(buffer, 32, "%d", (byteSize * 100) / total);
			return buffer;
		}
	}

	largeInt i = 0;
	bool addUnits = true;

	if (unit == unitsBytes)
	{
		i = 0;
		addUnits = false;
	}
	else if (unit == unitsKB)
	{
		i = 1;
		addUnits = false;
	}
	else if (unit == unitsMB)
	{
		i = 2;
		addUnits = false;
	}
	else if (unit == unitsGB)
	{
		i = 3;
		addUnits = false;
	}
	else
	{
		if(byteSize & 0xF000000000000000) i++;
		if(byteSize & 0xFFFC000000000000) i++;
		if(byteSize & 0xFFFFFF0000000000) i++;
		if(byteSize & 0xFFFFFFFFC0000000) i++;
		if(byteSize & 0xFFFFFFFFFFF00000) i++;
		if(byteSize & 0xFFFFFFFFFFFFFC00) i++;
	}

	largeInt divisor = 1 << (i * 10);
	largeInt quotient = byteSize / divisor;
	largeInt remainder = ((byteSize % divisor) * 100) / divisor;

	if (quotient > 9 && quotient < 100)
	{
		if ( remainder > 99 )
			remainder = remainder / 10;
	}
	else if (quotient >= 100)
	{
		remainder = 0;
	}

	if (remainder > 0)
		StringCchPrintf(buffer, 32, "%d.%02d", (int) quotient, (int) remainder);
	else
		StringCchPrintf(buffer, 32, "%d", (int) quotient);

	if (addUnits)
	{
		StringCchCat(buffer, 32, " ");
		StringCchCat(buffer, 32, units[i]);
	}

	return (string) buffer;
}

static inline int isSequenceChar(int ch)
{
	return (ch == 'd' || ch == 'm' || ch == 'y' || ch == 'h' || ch == 'i' || ch == 'n' || ch == 's');
}

string xStatsInfo::formatDateTime(const string &format, const SYSTEMTIME &st, int span)
{
	const char *formatPtr = format.c_str();
	string output;
	int lastChar = 0;
	int sequenceChar = 0;
	int count = 0;
	int ch = *formatPtr++;
	
	while(ch || sequenceChar)
	{
		if(sequenceChar)
		{
			if(ch == sequenceChar)
			{
				count++;
			}
			else if(isalpha(ch))
			{
				output.append(count, sequenceChar);
				output.append(1, ch);
				
				sequenceChar = 0;
				count = 0;
			}
			else
			{
				switch(sequenceChar)
				{
					case 'd':
					{
						switch(count)
						{
							case 1:
							{
								char temp[8];
								StringCchPrintf(temp, 8, "%d", st.wDay);
								output.append(temp);
								break;
							}
							
							case 2:
							{
								char temp[8];
								StringCchPrintf(temp, 8, "%02d", st.wDay);
								output.append(temp);
								break;
							}
							
							case 3:
							{
								if(st.wDayOfWeek < 7)
									output.append(shortWeekdays[st.wDayOfWeek]);
								
								break;
							}
							
							case 4:
							{
								if(st.wDayOfWeek < 7)
									output.append(weekdays[st.wDayOfWeek]);
									
								break;
							}
							
							default:
							{
								output.append(count, sequenceChar);
								break;
							}
						}
						
						break;
					}
					
					case 'm':
					{
						if(span) break;

						switch(count)
						{
							case 1:
							{
								char temp[8];
								StringCchPrintf(temp, 8, "%d", st.wMonth);
								output.append(temp);
								break;
							}
							
							case 2:
							{
								char temp[8];
								StringCchPrintf(temp, 8, "%02d", st.wMonth);
								output.append(temp);
								break;
							}
							
							case 3:
							{
								if(st.wMonth > 0 && st.wMonth < 13)
									output.append(shortMonths[st.wMonth - 1]);
								
								break;
							}
							
							case 4:
							{
								if(st.wMonth > 0 && st.wMonth < 13)
									output.append(months[st.wMonth - 1]);
								
								break;
							}
							
							default:
							{
								output.append(count, sequenceChar);
								break;
							}
						}
						
						break;
					}
					
					case 'y':
					{
						if(span) break;
					
						switch(count)
						{
							case 1:
							{	
								time_t now;
								tm* ptmTemp = NULL;

								char pszDateTime[128];
							
								// get the current time
								time(&now);
								ptmTemp = localtime(&now);

								if (ptmTemp == NULL || !strftime(pszDateTime, _countof(pszDateTime), "%j", ptmTemp))
									pszDateTime[0] = '\0';
								
								output.append(pszDateTime);

								break;
							}

							case 2:
							{
								char temp[8];
								StringCchPrintf(temp, 8, "%04d", st.wYear);
								output.append(temp + 2);

								break;
							}
						
							case 4:
							{
								char temp[8];
								StringCchPrintf(temp, 8, "%04d", st.wYear);
								output.append(temp);

								break;
							}

							default:
							{
								output.append(count, sequenceChar);
								break;
							}
						}
						
						break;
					}
					
					case 'h':
					{
						if(count == 1 || count == 2)
						{
							char temp[8];
							int hour = span ? st.wHour : (st.wHour == 0) ? 12 : (st.wHour > 12) ? st.wHour - 12 : st.wHour;
							StringCchPrintf(temp, 8, (count == 1) ? "%d" : "%02d", hour);
							output.append(temp);
						}
						else
						{
							output.append(count, sequenceChar);
						}
						
						break;
					}
					
					case 'i':
					{
						if(count == 1 || count == 2)
						{
							char temp[8];
							StringCchPrintf(temp, 8, (count == 1) ? "%d" : "%02d", st.wHour);
							output.append(temp);
						}
						else
						{
							output.append(count, sequenceChar);
						}
						
						break;
					}
					
					case 'n':
					{
						if(count == 1 || count == 2)
						{
							char temp[8];
							StringCchPrintf(temp, 8, (count == 1) ? "%d" : "%02d", st.wMinute);
							output.append(temp);
						}
						else
						{
							output.append(count, sequenceChar);
						}
						
						break;
					}
					
					case 's':
					{
						if(count == 1 || count == 2)
						{
							char temp[8];
							StringCchPrintf(temp, 8, (count == 1) ? "%d" : "%02d", st.wSecond);
							output.append(temp);
						}
						else
						{
							output.append(count, sequenceChar);
						}
						
						break;
					}
					
					default:
					{
						output.append(count, sequenceChar);
						break;
					}
				}
				
				if(ch) output.append(1, ch);
				sequenceChar = 0;
				count = 0;
			}
		}
		else
		{
			if(isSequenceChar(ch) && !isalpha(lastChar))
			{
				sequenceChar = ch;
				count = 1;
			}
			else if(ch == 'a' && !isalpha(lastChar))
			{
				const char *p = formatPtr - 1;
				
				// special case: am/pm
				if(_strnicmp(p, "am/pm", 5) == 0)
				{
					output.append((st.wHour >= 12) ? "pm" : "am");
					formatPtr = formatPtr + 4;
				}
				else
				{
					output.append(1, ch);
				}
			}
			else if(ch == 'A' && !isalpha(lastChar))
			{
				const char *p = formatPtr - 1;
				
				// special case: am/pm
				if(strncmp(p, "AM/PM", 5) == 0)
				{
					output.append((st.wHour >= 12) ? "PM" : "AM");
					formatPtr = formatPtr + 4;
				}
				else
				{
					output.append(1, ch);
				}
			}
			else
			{
				output.append(1, ch);
			}
		}
		
		lastChar = ch;
		if(ch) ch = *formatPtr++;
	}
	
	return output;
}

void xStatsInfo::getLocalizedTime(SYSTEMTIME *pst, const string &timezone)
{
	string temp = lowerCase(timezone);
	const char *s = temp.c_str();

	if(s[0] == 'u' && s[1] == 't' && s[2] == 'c')
	{
		s = s + 3;
		GetSystemTime(pst);
	}
	else
	{
		GetLocalTime(pst);
	}

	while(s[0] == ' ')
		s++;

	int dHour = 0;
	int dMinute = 0;
	int sign = 1;

	if(s[0] == '+')
	{
		sign = 1;
		s++;
	}
	else if(s[0] == '-')
	{
		sign = -1;
		s++;
	}

	while(s[0] >= '0' && s[0] <= '9')
	{
		dHour = (dHour * 10) + (s[0] - '0');
		s++;
	}

	if(s[0] == ':')
	{
		s++;

		while(s[0] >= '0' && s[0] <= '9')
		{
			dMinute = (dMinute * 10) + (s[0] - '0');
			s++;
		}
	}

	largeInt base;
	largeInt delta = (largeInt) (sign * ((dHour * 60) + dMinute)) * 600000000;

	SystemTimeToFileTime(pst, (LPFILETIME) &base);
	base = base + delta;
	FileTimeToSystemTime((LPFILETIME) &base, pst);
}

char *xStatsInfo::getLocaleInfo(LCTYPE type)
{
	int len = GetLocaleInfo(LOCALE_USER_DEFAULT, type, 0, 0);
	char *buffer = new char[len];
	GetLocaleInfo(LOCALE_USER_DEFAULT, type, buffer, len);
	return buffer;
}

string xStatsInfo::getClipboardText(bool *dynamic)
{
	if (dynamic) *dynamic = true;

	char buffer[MAX_LINE_LENGTH] = {0};
	GetClipboardText(NULL, buffer, MAX_LINE_LENGTH);

	return (string) buffer;
}

BOOL xStatsInfo::GetClipboardText(HWND hwndOwner, LPTSTR pszBuffer, UINT cchBuffer)
{
	HGLOBAL hMem;
	LPVOID pvMem;
	BOOL fResult = FALSE;

	if(IsClipboardFormatAvailable(CF_TEXT))
	{
		if(OpenClipboard(hwndOwner))
		{
			hMem = GetClipboardData(CF_TEXT);

			if(hMem)
			{
				pvMem = GlobalLock(hMem);
				lstrcpyn(pszBuffer, (LPCTSTR) pvMem, cchBuffer);
				GlobalUnlock(hMem);
				fResult = TRUE;
			}

			CloseClipboard();
		}
	}

	return fResult;
}

string xStatsInfo::iGetxRCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	char szValue[MAX_LINE_LENGTH] = {0};
	GetRCLine((uniqueKey + baseName).c_str(), szValue, MAX_LINE_LENGTH, defaultVal.c_str());

	return (string) szValue;
}

string xStatsInfo::IntToString(int input)
{
	char buffer[32] = {0};
	itoa( input, buffer, 10 );
	return (string) buffer;
}

bool xStatsInfo::IsOS(const int &os)
{	
	static int osversion = 0;

	static bool bInitialized = false;
	
	if (!bInitialized)
	{
		BOOL bOsVersionInfoEx;
		OSVERSIONINFOEX verInfo;
		ZeroMemory(&verInfo, sizeof(OSVERSIONINFOEX));
		verInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

		PGPI pGPI = NULL;
		DWORD dwType;

		if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &verInfo)) )
		{
			  verInfo.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
			  if (! GetVersionEx ( (OSVERSIONINFO *) &verInfo) ) 
			     return FALSE;
		}

		switch (verInfo.dwPlatformId)
		{
			// Test for the Windows NT product family.
			case VER_PLATFORM_WIN32_NT:

				osversion |= OS_NT;
			 
				// Test for the specific product.
				if ( verInfo.dwMajorVersion == 6 && verInfo.wProductType == VER_NT_WORKSTATION )
				{
					osversion |= OS_2KXP;
					osversion |= OS_XP;
					OSVersion = "Windows Vista";
					OSExtendedVersion = "Windows Vista ";
				}

				if ( verInfo.dwMajorVersion == 6 && (verInfo.wProductType == VER_NT_SERVER || verInfo.wProductType == VER_NT_DOMAIN_CONTROLLER) )
				{
					osversion |= OS_2KXP;
					osversion |= OS_XP;
					OSVersion = "Windows Server 2008";
					OSExtendedVersion = "Windows Server 2008, ";
				}

				if ( verInfo.dwMajorVersion == 5 && verInfo.dwMinorVersion == 2 )
				{
					osversion |= OS_2KXP;
					osversion |= OS_XP;
					OSVersion = "Windows Server 2003";
					OSExtendedVersion = "Windows Server 2003, ";
				}

				if ( verInfo.dwMajorVersion == 5 && verInfo.dwMinorVersion == 1 )
				{
					osversion |= OS_2KXP;
					osversion |= OS_XP;
					OSVersion = "Windows XP";
					OSExtendedVersion = "Windows XP ";
				}

				if ( verInfo.dwMajorVersion == 5 && verInfo.dwMinorVersion == 0 )
				{
					osversion |= OS_2KXP;
					osversion |= OS_2K;
					OSVersion = "Windows 2000";
					OSExtendedVersion = "Windows 2000 ";
				}

				if ( verInfo.dwMajorVersion <= 4 )
				{
					osversion |= OS_NT4;
					OSVersion = "Windows NT";
					OSExtendedVersion = "Windows NT ";
				}

				// Test for specific product on Windows NT 4.0 SP6 and later.
				if( bOsVersionInfoEx )
				{
					if ( verInfo.dwMajorVersion == 6 )
					{
						pGPI = (PGPI) GetProcAddress( GetModuleHandle(TEXT("kernel32.dll")), "GetProductInfo");

						if ( pGPI != NULL )
						{
							pGPI( 6, 0, 0, 0, &dwType);

							 switch( dwType )
							 {
								case PRODUCT_ULTIMATE:
								   OSExtendedVersion.append( "Ultimate Edition" );
								   break;
								case PRODUCT_HOME_PREMIUM:
								   OSExtendedVersion.append( "Home Premium Edition" );
								   break;
								case PRODUCT_HOME_BASIC:
								   OSExtendedVersion.append( "Home Basic Edition" );
								   break;
								case PRODUCT_ENTERPRISE:
								   OSExtendedVersion.append( "Enterprise Edition" );
								   break;
								case PRODUCT_BUSINESS:
								   OSExtendedVersion.append( "Business Edition" );
								   break;
								case PRODUCT_STARTER:
								   OSExtendedVersion.append( "Starter Edition" );
								   break;
								case PRODUCT_CLUSTER_SERVER:
								   OSExtendedVersion.append( "Cluster Server Edition" );
								   break;
								case PRODUCT_DATACENTER_SERVER:
								   OSExtendedVersion.append( "Datacenter Edition" );
								   break;
								case PRODUCT_DATACENTER_SERVER_CORE:
								   OSExtendedVersion.append( "Datacenter Edition (core installation)" );
								   break;
								case PRODUCT_ENTERPRISE_SERVER:
								   OSExtendedVersion.append( "Enterprise Edition" );
								   break;
								case PRODUCT_ENTERPRISE_SERVER_CORE:
								   OSExtendedVersion.append( "Enterprise Edition (core installation)" );
								   break;
								case PRODUCT_ENTERPRISE_SERVER_IA64:
								   OSExtendedVersion.append( "Enterprise Edition for Itanium-based Systems" );
								   break;
								case PRODUCT_SMALLBUSINESS_SERVER:
								   OSExtendedVersion.append( "Small Business Server" );
								   break;
								case PRODUCT_SMALLBUSINESS_SERVER_PREMIUM:
								   OSExtendedVersion.append( "Small Business Server Premium Edition" );
								   break;
								case PRODUCT_STANDARD_SERVER:
								   OSExtendedVersion.append( "Standard Edition" );
								   break;
								case PRODUCT_STANDARD_SERVER_CORE:
								   OSExtendedVersion.append( "Standard Edition (core installation)" );
								   break;
								case PRODUCT_WEB_SERVER:
								   OSExtendedVersion.append( "Web Server Edition" );
								   break;
								default:
									break;
							 }
						}
					}
					// Test for the workstation type.
					else if ( verInfo.wProductType == VER_NT_WORKSTATION )
					{
						if( verInfo.dwMajorVersion == 4 )
							OSExtendedVersion.append( "Workstation 4.0" );
						else if( verInfo.dwMajorVersion == 5 && verInfo.wSuiteMask & VER_SUITE_PERSONAL )
							OSExtendedVersion.append( "Home Edition" );
						else if ( verInfo.dwMajorVersion == 5 )
							OSExtendedVersion.append( "Professional" );
					}
					// Test for the server type.
					else if ( verInfo.wProductType == VER_NT_SERVER || 
						verInfo.wProductType == VER_NT_DOMAIN_CONTROLLER )
					{
						if( verInfo.dwMajorVersion == 5 && verInfo.dwMinorVersion == 2 )
						{
							if( verInfo.wSuiteMask & VER_SUITE_DATACENTER )
								OSExtendedVersion.append( "Datacenter Edition" );
							else if( verInfo.wSuiteMask & VER_SUITE_ENTERPRISE )
								OSExtendedVersion.append( "Enterprise Edition" );
							else if ( verInfo.wSuiteMask == VER_SUITE_BLADE )
								OSExtendedVersion.append( "Web Edition" );
							else
								OSExtendedVersion.append( "Standard Edition" );
						}
						else if( verInfo.dwMajorVersion == 5 && verInfo.dwMinorVersion == 0 )
						{
							if( verInfo.wSuiteMask & VER_SUITE_DATACENTER )
								OSExtendedVersion.append( "Datacenter Server" );
							else if( verInfo.wSuiteMask & VER_SUITE_ENTERPRISE )
								OSExtendedVersion.append( "Advanced Server" );
							else
								OSExtendedVersion.append( "Server" );
						}
						else  // Windows NT 4.0 
						{
							if( verInfo.wSuiteMask & VER_SUITE_ENTERPRISE )
								OSExtendedVersion.append( "Server 4.0, Enterprise Edition" );
							else
								OSExtendedVersion.append( "Server 4.0" );
						}
					}
				}

			 break;

			// Test for the Windows Me/98/95.
			case VER_PLATFORM_WIN32_WINDOWS:

				osversion |= OS_9X;

				if (verInfo.dwMajorVersion == 4 && verInfo.dwMinorVersion == 0)
				{
					osversion |= OS_95;
					OSVersion = "Windows 95";
					OSExtendedVersion = "Windows 95 ";
					if ( verInfo.szCSDVersion[1] == 'C' || verInfo.szCSDVersion[1] == 'B' )
					{
						OSExtendedVersion.append( "OSR2" );
					}
				} 

				if (verInfo.dwMajorVersion == 4 && verInfo.dwMinorVersion == 10)
				{
					osversion |= OS_98;
					OSVersion = "Windows 98";
					OSExtendedVersion = "Windows 98 ";
					if ( verInfo.szCSDVersion[1] == 'A' )
					{
						OSExtendedVersion.append("SE" );
					}
				} 

				if (verInfo.dwMajorVersion == 4 && verInfo.dwMinorVersion == 90)
				{
					osversion |= OS_ME;
					OSVersion = "Windows Millennium Edition";
					OSExtendedVersion = "Windows Millennium Edition";
				} 
				break;

			default:
			break;
	   }

		bInitialized = true;
	}

	return (osversion & os) ? true : false;
}

bool xStatsInfo::IsAppWindow(const HWND &hWnd)
{
	if(!IsWindowVisible(hWnd))
		return false;

	if(GetWindowLong(hWnd, GWL_USERDATA) == MAGIC_DWORD)
		return false;

	long exStyle = GetWindowLong(hWnd, GWL_EXSTYLE);

	if(exStyle & WS_EX_APPWINDOW)
		return true;
	
	if(GetWindow(hWnd, GW_OWNER) != 0)
		return false;

	if(exStyle & WS_EX_TOOLWINDOW)
		return false;

	return true;
}

BOOL CALLBACK TaskEnumWindowsProc(HWND hwnd, LPARAM lParam)
{
	xStatsInfo *pxStatsInfo = (xStatsInfo *) lParam;

	if ( pxStatsInfo->IsAppWindow(hwnd) && (pxStatsInfo->bActiveTaskOnly ? !IsIconic(hwnd) : true) )
	{
		char pszWindowText[MAX_LINE_LENGTH] = {0};
		GetWindowText(hwnd, pszWindowText, sizeof(pszWindowText));

		pxStatsInfo->tasksList.insert(pxStatsInfo->tasksList.end(), pszWindowText);
	}

	return TRUE;
}

string xStatsInfo::UpdateTasks(string sep, bool bActiveTask)
{
	tasksList.clear();

	bActiveTaskOnly = bActiveTask;

	EnumWindows(TaskEnumWindowsProc, (LPARAM) this);

	string result = "";
	if ( !bActiveTask )
	{
		if ( tasksList.empty() )
			return "";

		for (StringListIterator tit = tasksList.begin(); tit != tasksList.end(); tit++)
		{
			result.append((*tit));
			tit++;
			if (tit != tasksList.end())
				result.append(sep);
			tit--;
		}
	}
	else if (tasksList.begin() != tasksList.end())
	{
		result.append( (*tasksList.begin()) );
	}

	return result;
}