/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2008 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(__xStatsInfo_H)
#define __xStatsInfo_H

#include "NetStats.h"
#include "Processor.h"
#include "CPU_Info.h"

typedef vector<NetStats *>NetStatsList;
typedef vector<NetStats *>::iterator NetStatsListIterator;

typedef unsigned __int64 largeInt;

// .alur
typedef struct {
	const char *filename;
    const char *metadata;
    char *ret;
    size_t retlen;
} extendedFileInfoStruct;
// .alur

class xStatsInfo
{
public:

	xStatsInfo();
	virtual ~xStatsInfo();

	string processFinalText(const string &text, bool *dynamic = NULL);

	bool IsAppWindow(const HWND &hWnd);

	StringList tasksList;
	bool bActiveTaskOnly;

private:

	bool bCPUStarted;
	bool bNetStatsStarted;

	NetStatsList netStatsList;

	CPUInfo *pCPUInfo;

	// .alur
	int winampCurrentTrack;
    string winampAlbum;
    string winampAlbumTrack;
    string winampGenre;
    string winampYear;
    string winampComment;
	// .alur

	string evaluateFunction(const string &functionName, const vector<string> &arguments, bool *dynamic);
	string process(const string &text, int &i, int length, bool *dynamic);

	// if/elseif/else/endif
	void doIf(const vector<string> &arguments);
	void doIfEval(const vector<string> &arguments);
	void doElseIf(const vector<string> &arguments);
	void doElseIfEval(const vector<string> &arguments);
	void doElse();
	void doEndIf();

	// date and time info
	string getDate(const vector<string> &arguments, bool *dynamic);
	string getInternetTime(bool *dynamic);
	string getTime(const vector<string> &arguments, bool *dynamic);
	string getUptime(const vector<string> &arguments, bool *dynamic);

	// memory and disk space info
	string getDiskAvailable(const vector<string> &arguments, bool *dynamic);
	string getDiskInUse(const vector<string> &arguments, bool *dynamic);
	string getDiskTotal(const vector<string> &arguments, bool *dynamic);
	string getMemAvailable(const vector<string> &arguments, bool *dynamic);
	string getMemInUse(const vector<string> &arguments, bool *dynamic);
	string getMemTotal(const vector<string> &arguments, bool *dynamic);
	string getSwapAvailable(const vector<string> &arguments, bool *dynamic);
	string getSwapInUse(const vector<string> &arguments, bool *dynamic);
	string getSwapTotal(const vector<string> &arguments, bool *dynamic);

	// power info
	string getPowerSource(const vector<string> &arguments, bool *dynamic);
	string getBattery(bool *dynamic);

	// network-related info
	string getComputerName(bool *dynamic);
	string getConnected(bool *dynamic);
	string getHostName(bool *dynamic);
	string getIP(const vector<string> &arguments, bool *dynamic);
	string getOnline(const vector<string> &arguments, bool *dynamic);

	string getNetAdapterName(const vector<string> &arguments, bool *dynamic);

	string getNetIn(const vector<string> &arguments, bool *dynamic);
	string getNetOut(const vector<string> &arguments, bool *dynamic);
	string getNetInOut(const vector<string> &arguments, bool *dynamic);

	string getNetTotalIn(const vector<string> &arguments, bool *dynamic);
	string getNetTotalOut(const vector<string> &arguments, bool *dynamic);
	string getNetTotalInOut(const vector<string> &arguments, bool *dynamic);

	string getUserName(bool *dynamic);

	// Winamp info
	string getWinampSong(const vector<string> &arguments, bool *dynamic);
	string getWinampArtist(const vector<string> &arguments, bool *dynamic);
	string getWinampTitle(const vector<string> &arguments, bool *dynamic);
	string getWinampTime(const vector<string> &arguments, bool *dynamic);
	string getWinampRemainTime(const vector<string> &arguments, bool *dynamic);
	string getWinampTotalTime(const vector<string> &arguments, bool *dynamic);
	string getWinampStatus(const vector<string> &arguments, bool *dynamic);
	string getWinampBitRate(bool *dynamic);
	string getWinampSampleRate(bool *dynamic);
	string getWinampPlaying(bool *dynamic);
	string getWinampPaused(bool *dynamic);
	string getWinampStopped(bool *dynamic);
	string getWinampRepeat(bool *dynamic);
	string getWinampShuffle(bool *dynamic);
	// .alur
    string getExtendedWinampFileInfo(char* file, const char* field, HANDLE hWinamp, HWND hwndWA2);
    string getWinampInfo(int iWhichInfo, bool *dynamic);
    string getWinampRating(bool *dynamic);
	// .alur

	// MotherBoard Monitor 5 (MBM5) info
	string getMBMData(const char, const vector<string> &arguments, bool *dynamic);
	string getMBMLoaded(bool *dynamic);
	
	// file info
	string getDateCreated(const vector<string> &arguments, bool *dynamic);
	string getDateLastModified(const vector<string> &arguments, bool *dynamic);
	string getFileExists(const string &file, bool *dynamic);
	string getFirstLine(const vector<string> &arguments, bool *dynamic);
	string getLastLine(const vector<string> &arguments, bool *dynamic);
	string getLine(const vector<string> &arguments, bool *dynamic);
	string getMatchLine(const vector<string> &arguments, bool *dynamic);
	string getLineCount(const string &file, bool *dynamic);
	string getRandomLine(const string &file, bool *dynamic);
	string getSize(const vector<string> &arguments, bool *dynamic);

	// miscellaneous info
	string getMousePos(bool *dynamic);
	string getOS(bool *dynamic);
	string getOSEx(bool *dynamic);
	string getWindowTitle(const string &windowClass, bool *dynamic);
	string getClipboardText(bool *dynamic);
	string getVolume(const vector<string> &arguments, bool *dynamic);
	string getMute(bool *dynamic);

	//CPU Extended Info
	string getCPU(const vector<string> &arguments, bool *dynamic);
	string getCPUSpeed(const vector<string> &arguments, bool *dynamic);
	string getCPUInfo(const vector<string> &arguments, bool *dynamic);

	// formatting functions
	string capitalize(const string &s);
	string after(const string &s, const string &search);
	string afterLast(const string &s, const string &search);
	string before(const string &s, const string &search);
	string beforeLast(const string &s, const string &search);
	string between(const string &s, const string &left, const string &right, const string &modifier = "");
	string trim(const string &s, const string &search = " ");
	string remove(const string &s, const string &search = "");
	string replace(const string &s, const string &search, const string &replace);
	string lowerCase(const string &s);
	string upperCase(const string &s);
	string verticaldown(const string &s);
	string verticalup(const string &s);

	// miscellaneous functions
	string exportedEvars(const string &s, bool *dynamic);
	string empty(const string &s, bool *dynamic);
	string notEmpty(const string &s, bool *dynamic);

	void getDiskFreeSpace(const string &drive, largeInt &freeBytes, largeInt &totalBytes);

	string formatByteSize(largeInt byteSize, largeInt total, int units);
	string formatDateTime(const string &format, const SYSTEMTIME &st, int span);
	void getLocalizedTime(SYSTEMTIME *pst, const string &timezone);

	char *getLocaleInfo(LCTYPE type);

	string getActiveTask(const vector<string> &arguments, bool *dynamic);
	string getTasks(const vector<string> &arguments, bool *dynamic);

	string UpdateTasks(string sep, bool bActiveTask = false);

private:

	string getKeyboardLayoutShortName(bool *dynamic);
	string getKeyboardLayoutName(bool *dynamic);
	string getKeyboardState(const vector<string> &arguments, bool *dynamic);

	vector<bool> condStack;
	vector<int> depthStack;
	vector<bool> maskStack;

	char *shortWeekdays[7];
	char *weekdays[7];
	char *shortMonths[12];
	char *months[12];

private:

	//Helper Functions
	bool IsOS(const int &os);
		string OSVersion;
		string OSExtendedVersion;

	string iGetxRCLine(const string &uniqueKey, const string &baseName, const string &defaultVal);

	string IntToString(int input);

	BOOL GetClipboardText(HWND hwndOwner, LPTSTR pszBuffer, UINT cchBuffer);

};

#endif
